class SheetPanel < ActiveRecord::Base
  belongs_to :author
  belongs_to :panel
  belongs_to :sheet
  
  validates :sheet_id, :presence => true, :numericality => true, :existence => {:both => false}
  validates :panel_id, :presence => true, :numericality => true, :existence => {:both => false}
  validates :author_id, :presence => true, :numericality => true, :existence => {:both => false}
  validates :x, :presence => true, :numericality => true
  validates :y, :presence => true, :numericality => true
  validates :z, :presence => true, :numericality => {:greater_than => 0}
  validates :t, :presence => true, :numericality => {:greater_than_or_equal_to => 0}
  
  def supply_default
    self.sheet_id = nil
    self.panel_id = nil
    self.t = nil
  end
  
  def overwrite au
    return false unless au
    self.author_id = au.id
  end
  
  def own? roles
    roles = [roles] unless roles.respond_to?(:each)
    au = SheetPanel.get_author_from_roles roles
    return false unless au
    self.author_id == au.id
  end
  
  def visible? roles
    if MagicNumber['run_mode'] == 0
      return false unless guest_role_check(roles)
    else
      return false unless reader_role_check(roles)
    end
    return true if self.sheet.own?(roles)
    self.sheet.visible? roles
  end
  
  def self.default_panel_size
    30
  end
  
  def self.max_panel_size
    200
  end
  
  def self.offset cnt, prm = nil
    offset = prm.to_i
    offset = cnt - 1 if offset >= cnt
    offset = cnt - offset.abs if offset < 0
    offset = 0 if offset < 0
    offset
  end
  
  def self.panel_count cnt, prm = self.default_panel_size
    count = prm.to_i
    count = self.max_panel_size if count > self.max_panel_size
    count = self.default_panel_size if count < 1
    count
  end
  
  def self.default_page_size
    25
  end
  
  def self.max_page_size
    100
  end
  
  def self.page prm = nil
    page = prm.to_i
    page = 1 if page < 1
    page
  end
  
  def self.page_size prm = self.default_page_size
    page_size = prm.to_i
    page_size = self.max_page_size if page_size > self.max_page_size
    page_size = self.default_page_size if page_size < 1
    page_size
  end
  
  def self.play_list_where cid
    ['sheet_panels.sheet_id = ?', cid]
  end
  
  def self.list_where
    'sheets.visible > 0'
  end
  
  def self.mylist_where au
    ['sheet_panels.author_id = ?', au.id]
  end
  
  def self.himlist_where au
    ['sheet_panels.author_id = ? and sheets.visible > 0', au.id]
  end
  
  def self.play_list sheet, author, offset = 0, limit = SheetPanel.default_panel_size
    SheetPanel.where(self.play_list_where(sheet.id)).includes(SheetPanel.list_opt).order('sheet_panels.t').offset(offset).limit(limit)
  end
  
  def self.list page = 1, page_size = self.default_page_size
    SheetPanel.where(self.list_where()).includes(SheetPanel.list_opt).order('sheet_panels.updated_at desc').offset((page -1) * page_size).limit(page_size)
  end
  
  def self.mylist au, page = 1, page_size = Author.default_sheet_panel_page_size
    SheetPanel.where(self.mylist_where(au)).includes(SheetPanel.list_opt).order('sheet_panels.updated_at desc').offset((page -1) * page_size).limit(page_size)
  end
  
  def self.himlist au, page = 1, page_size = Author.default_sheet_panel_page_size
    SheetPanel.where(self.himlist_where(au)).includes(SheetPanel.list_opt).order('sheet_panels.updated_at desc').offset((page -1) * page_size).limit(page_size)
  end
  
  def self.list_paginate page = 1, page_size = self.default_page_size
    Kaminari.paginate_array(Array.new(SheetPanel.where(self.list_where()).includes(SheetPanel.list_opt).count, nil)).page(page).per(page_size)
  end
  
  def self.mylist_paginate au, page = 1, page_size = Author.default_sheet_panel_page_size
    Kaminari.paginate_array(Array.new(SheetPanel.where(self.mylist_where(au)).includes(SheetPanel.list_opt).count, nil)).page(page).per(page_size)
  end
  
  def self.himlist_paginate au, page = 1, page_size = Author.default_sheet_panel_page_size
    Kaminari.paginate_array(Array.new(SheetPanel.where(self.himlist_where(au)).includes(SheetPanel.list_opt).count, nil)).page(page).per(page_size)
  end
  
  def self.list_opt
    {
      :author => {}, 
      :sheet => {
        :author => {}
      }, 
      :panel => {
        :author => {}, 
        :panel_pictures => {:picture => {:artist => {}, :license => {}}}, 
        :speech_balloons =>{:balloon => {}, :speech => {}}
      }
    }
  end
  
  def self.list_json_opt
    {:include => {
      :author => {}, 
      :sheet => {
        :author => {}
      }, 
      :panel => {
        :author => {}, 
        :panel_pictures => {:picture => {:artist => {}, :license => {}}}, 
        :speech_balloons =>{:balloon => {}, :speech => {}}
      }
    }}
  end
  
  def self.show spid, roles
    opt = {}
    opt.merge!(SheetPanel.show_opt)
    res = SheetPanel.find spid, opt
    raise ActiveRecord::Forbidden unless res.visible?(roles)
    res
  end
  
  def self.edit spid, au
    opt = {}
    opt.merge!(SheetPanel.show_opt)
    res = SheetPanel.find spid, opt
    raise ActiveRecord::Forbidden unless res.own?(au)
    res
  end
  
  def self.show_opt
    {:include => {
      :author => {}, 
      :sheet => {
        :author => {}
      }, 
      :panel => {
        :author => {}, 
        :panel_pictures => {:picture => {:artist => {}, :license => {}}}, 
        :speech_balloons =>{:balloon => {}, :speech => {}}
      }
    }}
  end
  
  def elements
    self.panel.elements
  end
  
  def sheet_panel_as_json au
    panel_include = if self.panel and self.panel.visible?(au)
      {:include => {:author => {}}, :methods => :elements}
    else
      {:include => {:author => {}}}
    end
    self.to_json({:include => {:sheet => {:include => {:author => {}}}, :author => {}, :panel => panel_include}})
  end
  
  def self.list_as_json_text ary, au
    '[' + ary.map {|i| i.sheet_panel_as_json(au) }.join(',') + ']'
  end
  
  def self.licensed_pictures sheet_panels
    r = {}
    sheet_panels.each do |sheet_panel|
      r.merge!(sheet_panel.panel.licensed_pictures) if sheet_panel.panel
    end
    r
  end
  
  def self.new_t sheet_id
    r = SheetPanel.max_t(sheet_id)
    r.blank? ? 0 : r.to_i + 1
  end
  
  def self.max_t sheet_id
    SheetPanel.maximum(:t, :conditions => ['sheet_id = ?', sheet_id])
  end
  
  def self.find_t sheet_id, t
    SheetPanel.find(:first, :conditions => ['sheet_id = ? and t = ?', sheet_id, t])
  end
  
  def self.collect_t sheet_panel
    r = SheetPanel.find(:all, :conditions => ['sheet_id = ?', sheet_panel.sheet_id], :order => 't')
    r.map {|sp| sp.t}
  end
  
  def self.serial? ary
    i = 0
    ary.compact.sort.each do |t|
      break false unless t == i
      i += 1
    end
    ary.compact.size == i
  end
  
  def self.validate_t sheet_panel
    SheetPanel.serial?(SheetPanel.collect_t(sheet_panel))
  end
  
  def insert_shift
    SheetPanel.update_all('t = t + 1', ['sheet_id = ? and t >= ?', self.sheet_id, self.t])
  end
  
  def lesser_shift old_t
    self.t = 0 if self.t < 0
    SheetPanel.update_all('t = t + 1', ['sheet_id = ? and (t >= ? and t < ?)', self.sheet_id, self.t, old_t])
  end
  
  def higher_shift old_t
    nf = SheetPanel.find_t(self.sheet_id, self.t)
    max_t = SheetPanel.max_t(self.sheet_id).to_i
    self.t = max_t if self.t > max_t
    SheetPanel.update_all('t = t - 1', ['sheet_id = ? and (t > ? and t <= ?)', self.sheet_id, old_t, self.t])
  end
  
  def update_shift old_t
    if self.t > old_t
      higher_shift old_t
    else
      lesser_shift old_t
    end
  end
  
  def rotate old_t = nil
    if self.new_record?
      if self.t.blank?
        self.t = SheetPanel.new_t self.sheet_id
      else
        self.insert_shift
      end
    else
      if self.t.blank?
      else
        self.update_shift old_t
      end
    end
  end
  
  def allow?
    return nil if self.sheet_id == nil or self.panel_id == nil
    self.sheet.own?(self.author) and self.panel.usable?(self.author)
  end
  
  def store old_t = nil
    res = false
    SheetPanel.transaction do
      case self.allow?
      when true
        self.rotate old_t
      when false
        raise ActiveRecord::Forbidden
      else
      end
      res = self.save
      raise ActiveRecord::Rollback unless res
      res = SheetPanel.validate_t(self) 
      unless res
        self.errors.add :t, 'unserialized'
        raise ActiveRecord::Rollback 
      end
    end
    res
  end
  
  def destroy_and_shorten
    res = false
    SheetPanel.transaction do
      SheetPanel.update_all('t = t - 1', ['sheet_id = ? and (t > ?)', self.sheet_id, self.t])
      raise ActiveRecord::Rollback unless self.destroy
      res = true
    end
    res
  end
  
end
