# -*- encoding: utf-8 -*-
require 'spec_helper'
#スクロール

describe Scroll do
  before do
    SpeechBalloonTemplate.delete_all
    @admin = FactoryGirl.create :admin
    @demand_user = FactoryGirl.create :demand_user
    @sp = FactoryGirl.create :system_picture
    @lg = FactoryGirl.create :license_group
    @license = FactoryGirl.create :license, :license_group_id => @lg.id, :system_picture_id => @sp.id
    @user = FactoryGirl.create( :user_yas)
    @author = FactoryGirl.create :author, :user_id => @user.id
    @artist = FactoryGirl.create :artist_yas, :author_id => @author.id
    @other_user = FactoryGirl.create( :user_yas)
    @other_author = FactoryGirl.create :author, :user_id => @other_user.id
    @other_artist = FactoryGirl.create :artist_yas, :author_id => @other_author.id
  end
  
  describe '検証に於いて' do
    before do
      @scroll = FactoryGirl.build :scroll, :author_id => @author.id
    end
    
    context 'オーソドックスなデータのとき' do
      it '下限データが通る' do
        @scroll.title = 'a'
        @scroll.visible = 0
        @scroll.should be_valid
      end
      it '上限データが通る' do
        @scroll.title = 'a'*100
        @scroll.visible = 1
        @scroll.should be_valid
      end
    end
    
    context 'titleを検証するとき' do
      it 'nullなら失敗する' do
        @scroll.title = nil
        @scroll.should_not be_valid
      end
      it '100文字以上なら失敗する' do
        @scroll.title = 'a'*101
        @scroll.should_not be_valid
      end
    end
    context 'visibleを検証するとき' do
      it 'nullなら失敗する' do
        @scroll.visible = nil
        @scroll.should_not be_valid
      end
      it '負なら失敗する' do
        @scroll.visible = -1
        @scroll.should_not be_valid
      end
      it '2以上なら失敗する' do
        @scroll.visible = 2
        @scroll.should_not be_valid
      end
    end
  end
  
  describe '文字コード検証に於いて' do
    before do
      @scroll = FactoryGirl.build :scroll, :author_id => @author.id
    end
    
    context 'titleを検証するとき' do
      it 'Shift JISなら失敗する' do
        @scroll.title = "\x83G\x83r\x83]\x83D"
        lambda{
          @scroll.valid_encode
        }.should raise_error(Pettanr::BadRequest)
      end
    end
    
    context 'descriptionを検証するとき' do
      it 'Shift JISなら失敗する' do
        @scroll.description = "\x83G\x83r\x83]\x83D"
        lambda{
          @scroll.valid_encode
        }.should raise_error(Pettanr::BadRequest)
      end
    end
  end
  
  describe 'デフォルト値補充に於いて' do
    it 'visibleが0になっている' do
      @scroll = FactoryGirl.build :scroll, :visible => nil
      @scroll.supply_default
      @scroll.visible.should eq 0
    end
  end
  
  describe '上書き補充に於いて' do
    it '作家idが設定されている' do
      @scroll = FactoryGirl.build :scroll, :author_id => nil
      @scroll.overwrite @author
      @scroll.author_id.should eq @author.id
    end
  end
  
  describe 'ロールリストからの作家取得に於いて' do
    context 'ロールがユーザのとき' do
      it 'ユーザから作家を取得して、それを返す' do
        r = Scroll.get_author_from_roles([@user])
        r.should eq @author
      end
    end
    context 'ロールが作家のとき' do
      it '作家を返す' do
        r = Scroll.get_author_from_roles([@author])
        r.should eq @author
      end
    end
    context 'ロールが絵師のとき' do
      it '絵師から作家を取得できれば、それをを返す' do
        r = Scroll.get_author_from_roles([@artist])
        r.should eq @author
      end
      it '絵師から作家を取得できないときnilを返す' do
        @artist.author_id = nil
        @artist.save!
        r = Scroll.get_author_from_roles([@artist])
        r.should be_nil
      end
    end
    context 'ロールが管理者のとき' do
      it 'nilを返す' do
        r = Scroll.get_author_from_roles([@admin])
        r.should be_nil
      end
    end
  end
  
  describe 'ロールリストからの絵師取得に於いて' do
    context 'ロールがユーザのとき' do
      it 'ユーザから作家を取得して、そこから絵師を取得できれば、それを返す' do
        r = Scroll.get_artist_from_roles([@user])
        r.should eq @artist
      end
      it '作家から絵師を取得できないときnilを返す' do
        @artist.author_id = nil
        @artist.save!
        r = Scroll.get_artist_from_roles([@user])
        r.should be_nil
      end
    end
    context 'ロールが作家のとき' do
      it 'そこから絵師を取得できれば、それを返す' do
        r = Scroll.get_artist_from_roles([@author])
        r.should eq @artist
      end
      it '作家から絵師を取得できないときnilを返す' do
        @artist.author_id = nil
        @artist.save!
        r = Scroll.get_artist_from_roles([@author])
        r.should be_nil
      end
    end
    context 'ロールが絵師のとき' do
      it 'それを返す' do
        r = Scroll.get_artist_from_roles([@artist])
        r.should eq @artist
      end
    end
    context 'ロールが管理者のとき' do
      it 'nilを返す' do
        r = Scroll.get_artist_from_roles([@admin])
        r.should be_nil
      end
    end
  end
  
  describe '所持判定に於いて' do
    before do
      @scroll = FactoryGirl.build :scroll, :author_id => @author.id
    end
    context '事前チェックする' do
      it '自身にロールリストからの作家取得を依頼している' do
        Scroll.should_receive(:get_author_from_roles).with(any_args).exactly(1)
        r = @scroll.own?([@author])
      end
    end
    context 'ロール内作家が取得できるとき' do
      before do
      end
      it 'ロール内作家のidが自身の作家idと一致するなら許可する' do
        Scroll.stub(:get_author_from_roles).with(any_args).and_return(@author)
        r = @scroll.own?([@author])
        r.should be_true
      end
      it 'ロール内作家のidが自身の作家idと一致しないならno' do
        Scroll.stub(:get_author_from_roles).with(any_args).and_return(@other_author)
        @scroll.own?(@other_author).should be_false
      end
    end
    context 'ロール内作家が取得できないとき' do
      before do
        Scroll.stub(:get_author_from_roles).with(any_args).and_return(nil)
      end
      it 'Falseを返す' do
        r = @scroll.own?([@author])
        r.should be_false
      end
    end
  end
  
  describe '読者用ロールチェックに於いて' do
    before do
      @scroll = FactoryGirl.build :scroll, :author_id => @author.id
    end
    context 'ロールリストに作家が含まれるとき' do
      it 'ロールリストがリストではないとき、リストにする trueを返す' do
        r = @scroll.reader_role_check(@author)
        r.should be_true
      end
      it 'trueを返す' do
        r = @scroll.reader_role_check([@author])
        r.should be_true
      end
    end
    context 'ロールリストに絵師が含まれるとき' do
      it 'trueを返す' do
        r = @scroll.reader_role_check([@artist])
        r.should be_true
      end
    end
    context 'ロールリストにユーザが含まれるとき' do
      it 'trueを返す' do
        r = @scroll.reader_role_check([@user])
        r.should be_true
      end
    end
    context 'ロールリストに管理者が含まれるとき' do
      it 'trueを返す' do
        r = @scroll.reader_role_check([@admin])
        r.should be_true
      end
    end
    context 'ロールリストにユーザ、管理者、作家、絵師が含まれないとき' do
      it 'falseを返す' do
        r = @scroll.reader_role_check([nil])
        r.should be_false
      end
    end
  end
  
  describe '素材読者用ロールチェックに於いて' do
    before do
      @scroll = FactoryGirl.build :scroll, :author_id => @author.id
    end
    context 'ロールリストに作家が含まれるとき' do
      it 'ロールリストがリストではないとき、リストにする trueを返す' do
        r = @scroll.resource_reader_role_check(@author)
        r.should be_true
      end
      it 'trueを返す' do
        r = @scroll.resource_reader_role_check([@author])
        r.should be_true
      end
    end
    context 'ロールリストに絵師が含まれるとき' do
      it 'trueを返す' do
        r = @scroll.resource_reader_role_check([@artist])
        r.should be_true
      end
    end
    context 'ロールリストにユーザが含まれるとき' do
      it 'trueを返す' do
        r = @scroll.resource_reader_role_check([@user])
        r.should be_true
      end
    end
    context 'ロールリストに管理者が含まれるとき' do
      it 'trueを返す' do
        r = @scroll.resource_reader_role_check([@admin])
        r.should be_true
      end
    end
    context 'ロールリストに借手が含まれるとき' do
      it 'trueを返す' do
        r = @scroll.resource_reader_role_check([@demand_user])
        r.should be_true
      end
    end
    context 'ロールリストにユーザ、管理者、作家、絵師、借手が含まれないとき' do
      it 'falseを返す' do
        r = @scroll.resource_reader_role_check([nil])
        r.should be_false
      end
    end
  end
  
  describe '閲覧許可に於いて' do
    before do
      @scroll = FactoryGirl.build :scroll, :author_id => @author.id
    end
    context 'オープンモードのとき' do
      before do
        MagicNumber['run_mode'] = 0
      end
      it '自身にゲスト用ロールチェックを問い合わせしている' do
        Scroll.any_instance.stub(:guest_role_check).and_return(true)
        Scroll.any_instance.should_receive(:guest_role_check).with(any_args).exactly(1)
        r = @scroll.visible?([@author])
      end
      it 'ゲスト用ロールチェックが失敗したとき、falseを返す' do
        Scroll.any_instance.stub(:guest_role_check).and_return(false)
        r = @scroll.visible?([@author])
        r.should be_false
      end
    end
    context 'クローズドモードのとき' do
      before do
        MagicNumber['run_mode'] = 1
      end
      it '自身に読者用ロールチェックを問い合わせしている' do
        Scroll.any_instance.stub(:reader_role_check).and_return(true)
        Scroll.any_instance.should_receive(:reader_role_check).with(any_args).exactly(1)
        r = @scroll.visible?([@author])
      end
      it '読者用ロールチェックが失敗したとき、falseを返す' do
        Scroll.any_instance.stub(:reader_role_check).and_return(false)
        r = @scroll.visible?([@author])
        r.should be_false
      end
    end
    context '事前チェックする' do
      before do
        MagicNumber['run_mode'] = 1
        Scroll.any_instance.stub(:reader_role_check).and_return(true)
        Scroll.any_instance.stub(:own?).and_return(true)
      end
      it '自身に所持判定を問い合わせしている' do
        Scroll.any_instance.should_receive(:own?).with(any_args).exactly(1)
        r = @scroll.visible?([@author])
      end
    end
    context 'つつがなく終わるとき' do
      before do
        MagicNumber['run_mode'] = 1
        Scroll.any_instance.stub(:reader_role_check).and_return(true)
      end
      it '自分のスクロールなら許可する' do
        Scroll.any_instance.stub(:own?).and_return(true)
        Scroll.any_instance.stub(:visible).and_return(0)
        r = @scroll.visible?([@author])
        r.should be_true
      end
      it '他人の非公開スクロールなら許可しない' do
        Scroll.any_instance.stub(:own?).and_return(false)
        Scroll.any_instance.stub(:visible).and_return(0)
        r = @scroll.visible?([@author])
        r.should be_false
      end
      it '他人のスクロールでも公開なら許可する' do
        Scroll.any_instance.stub(:own?).and_return(false)
        Scroll.any_instance.stub(:visible).and_return(1)
        r = @scroll.visible?([@author])
        r.should be_true
      end
    end
  end
  
  describe '一覧取得に於いて' do
    before do
      @scroll = FactoryGirl.create :scroll, :author_id => @author.id
    end
    context 'page補正について' do
      it '文字列から数値に変換される' do
        Scroll.page('8').should eq 8
      end
      it 'nilの場合は1になる' do
        Scroll.page().should eq 1
      end
      it '0以下の場合は1になる' do
        Scroll.page('0').should eq 1
      end
    end
    context 'page_size補正について' do
      it '文字列から数値に変換される' do
        Scroll.page_size('7').should eq 7
      end
      it 'nilの場合はScroll.default_page_sizeになる' do
        Scroll.page_size().should eq Scroll.default_page_size
      end
      it '0以下の場合はScroll.default_page_sizeになる' do
        Scroll.page_size('0').should eq Scroll.default_page_size
      end
      it 'Scroll.max_page_sizeを超えた場合はScroll.max_page_sizeになる' do
        Scroll.page_size('1000').should eq Scroll.max_page_size
      end
    end
    context 'つつがなく終わるとき' do
      it '一覧取得オプションを利用している' do
        Scroll.stub(:list_opt).with(any_args).and_return({})
        Scroll.should_receive(:list_opt).with(any_args).exactly(1)
        r = Scroll.list
      end
    end
    it 'リストを返す' do
      c = Scroll.list
      c.should eq [@scroll]
    end
    it '非公開スクロールは(自分のスクロールであっても)含んでいない' do
      FactoryGirl.create :scroll, :author_id => @author.id, :visible => 0
      c = Scroll.list
      c.should eq [@scroll]
    end
    it '時系列で並んでいる' do
      #公開スクロールは(他人のスクロールであっても)含んでいる
      v = FactoryGirl.create :scroll, :author_id => @other_author.id, :updated_at => Time.now + 100
      c = Scroll.list
      c.should eq [v, @scroll]
    end
    context 'DBに5件あって1ページの件数を2件に変えたとして' do
      before do
        @scroll2 = FactoryGirl.create :scroll, :author_id => @author.id, :updated_at => Time.now + 100
        @scroll3 = FactoryGirl.create :scroll, :author_id => @author.id, :updated_at => Time.now + 200
        @scroll4 = FactoryGirl.create :scroll, :author_id => @author.id, :updated_at => Time.now + 300
        @scroll5 = FactoryGirl.create :scroll, :author_id => @author.id, :updated_at => Time.now + 400
        Scroll.stub(:default_page_size).and_return(2)
      end
      it '通常は2件を返す' do
        c = Scroll.list
        c.should have(2).items 
      end
      it 'page=1なら末尾2件を返す' do
        #時系列で並んでいる
        c = Scroll.list(1)
        c.should eq [@scroll5, @scroll4]
      end
      it 'page=2なら中間2件を返す' do
        c = Scroll.list(2)
        c.should eq [@scroll3, @scroll2]
      end
      it 'page=3なら先頭1件を返す' do
        c = Scroll.list(3)
        c.should eq [@scroll]
      end
    end
  end
  
  describe '自分のスクロール一覧取得に於いて' do
    before do
      @scroll = FactoryGirl.create :scroll, :author_id => @author.id
    end
    context 'つつがなく終わるとき' do
      it '一覧取得オプションを利用している' do
        Scroll.stub(:list_opt).with(any_args).and_return({})
        Scroll.should_receive(:list_opt).with(any_args).exactly(1)
        r = Scroll.mylist @author
      end
    end
    it 'リストを返す' do
      c = Scroll.mylist @author
      c.should eq [@scroll]
    end
    it '時系列で並んでいる' do
      nc = FactoryGirl.create :scroll, :author_id => @author.id, :updated_at => Time.now + 100
      cl = Scroll.mylist @author
      cl.should eq [nc, @scroll]
    end
    it '他人のスクロールは公開でも含まない' do
      nc = FactoryGirl.create :scroll, :author_id => @other_author.id, :visible => 1
      cl = Scroll.mylist @author
      cl.should eq [@scroll]
    end
    it '自分のスクロールは非公開でも含んでいる' do
      nc = FactoryGirl.create :scroll, :author_id => @author.id, :visible => 0, :updated_at => Time.now + 100
      cl = Scroll.mylist @author
      cl.should eq [nc, @scroll]
    end
    context 'DBに5件あって1ページの件数を2件に変えたとして' do
      before do
        @scroll2 = FactoryGirl.create :scroll, :author_id => @author.id, :updated_at => Time.now + 100
        @scroll3 = FactoryGirl.create :scroll, :author_id => @author.id, :updated_at => Time.now + 200
        @scroll4 = FactoryGirl.create :scroll, :author_id => @author.id, :updated_at => Time.now + 300
        @scroll5 = FactoryGirl.create :scroll, :author_id => @author.id, :updated_at => Time.now + 400
      end
      it '通常は2件を返す' do
        c = Scroll.mylist @author, 1, 2
        c.should have(2).items 
      end
      it 'page=1なら末尾2件を返す' do
        #時系列で並んでいる
        c = Scroll.mylist(@author, 1, 2)
        c.should eq [@scroll5, @scroll4]
      end
      it 'page=2なら中間2件を返す' do
        c = Scroll.mylist(@author, 2, 2)
        c.should eq [@scroll3, @scroll2]
      end
      it 'page=3なら先頭1件を返す' do
        c = Scroll.mylist(@author, 3, 2)
        c.should eq [@scroll]
      end
    end
  end
  
  describe '他作家のスクロール一覧取得に於いて' do
    before do
      @scroll = FactoryGirl.create :scroll, :author_id => @author.id
      @other_scroll = FactoryGirl.create :scroll, :author_id => @other_author.id, :visible => 1
    end
    context 'つつがなく終わるとき' do
      it '一覧取得オプションを利用している' do
        Scroll.stub(:list_opt).with(any_args).and_return({})
        Scroll.should_receive(:list_opt).with(any_args).exactly(1)
        r = Scroll.himlist @other_author
      end
    end
    it '指定した作家のリストを返す' do
      r = Scroll.himlist @other_author
      r.should eq [@other_scroll]
    end
    it '時系列で並んでいる' do
      nc = FactoryGirl.create :scroll, :author_id => @other_author.id, :updated_at => Time.now + 100
      r = Scroll.himlist @other_author
      r.should eq [nc, @other_scroll]
    end
    it '公開スクロールに限る ' do
      hidden = FactoryGirl.create :scroll, :author_id => @other_author.id, :visible => 0
      r = Scroll.himlist @other_author
      r.should eq [@other_scroll]
    end
    context 'DBに5件あって1ページの件数を2件に変えたとして' do
      before do
        @other_scroll2 = FactoryGirl.create :scroll, :author_id => @other_author.id, :updated_at => Time.now + 100
        @other_scroll3 = FactoryGirl.create :scroll, :author_id => @other_author.id, :updated_at => Time.now + 200
        @other_scroll4 = FactoryGirl.create :scroll, :author_id => @other_author.id, :updated_at => Time.now + 300
        @other_scroll5 = FactoryGirl.create :scroll, :author_id => @other_author.id, :updated_at => Time.now + 400
      end
      it '通常は2件を返す' do
        c = Scroll.himlist @other_author, 1, 2
        c.should have(2).items 
      end
      it 'page=1なら末尾2件を返す' do
        #時系列で並んでいる
        c = Scroll.himlist(@other_author, 1, 2)
        c.should eq [@other_scroll5, @other_scroll4]
      end
      it 'page=2なら中間2件を返す' do
        c = Scroll.himlist(@other_author, 2, 2)
        c.should eq [@other_scroll3, @other_scroll2]
      end
      it 'page=3なら先頭1件を返す' do
        c = Scroll.himlist(@other_author, 3, 2)
        c.should eq [@other_scroll]
      end
    end
  end
  
  describe 'スクロール一覧ページ制御に於いて' do
    before do
      Scroll.stub(:count).with(any_args).and_return(100)
    end
    it 'ページ制御を返す' do
      r = Scroll.list_paginate 
      r.is_a?(Kaminari::PaginatableArray).should be_true
    end
    it 'スクロール一覧の取得条件を利用している' do
      Scroll.stub(:list_where).with(any_args).and_return('')
      Scroll.should_receive(:list_where).with(any_args).exactly(1)
      r = Scroll.list_paginate 
    end
    it 'ページ件数10のとき、3ページ目のオフセットは20から始まる' do
      r = Scroll.list_paginate 3, 10
      r.limit_value.should eq 10
      r.offset_value.should eq 20
    end
  end
  
  describe '自分のスクロール一覧ページ制御に於いて' do
    before do
      Scroll.stub(:count).with(any_args).and_return(100)
    end
    it 'ページ制御を返す' do
      r = Scroll.mylist_paginate @author
      r.is_a?(Kaminari::PaginatableArray).should be_true
    end
    it '自分のスクロール一覧の取得条件を利用している' do
      Scroll.stub(:mylist_where).with(any_args).and_return('')
      Scroll.should_receive(:mylist_where).with(any_args).exactly(1)
      r = Scroll.mylist_paginate @author
    end
    it 'ページ件数10のとき、3ページ目のオフセットは20から始まる' do
      r = Scroll.mylist_paginate @author, 3, 10
      r.limit_value.should eq 10
      r.offset_value.should eq 20
    end
  end
  
  describe '他作家のスクロール一覧ページ制御に於いて' do
    before do
      Scroll.stub(:count).with(any_args).and_return(100)
    end
    it 'ページ制御を返す' do
      r = Scroll.himlist_paginate @other_author
      r.is_a?(Kaminari::PaginatableArray).should be_true
    end
    it '他作家のスクロール一覧の取得条件を利用している' do
      Scroll.stub(:himlist_where).with(any_args).and_return('')
      Scroll.should_receive(:himlist_where).with(any_args).exactly(1)
      r = Scroll.himlist_paginate @other_author
    end
    it 'ページ件数10のとき、3ページ目のオフセットは20から始まる' do
      r = Scroll.himlist_paginate @other_author, 3, 10
      r.limit_value.should eq 10
      r.offset_value.should eq 20
    end
  end
  
  describe '一覧取得オプションに於いて' do
    it '2つの項目を含んでいる' do
      r = Scroll.list_opt
      r.should have(2).items
    end
    it 'スクコマを含んでいる' do
      r = Scroll.list_opt
      r.has_key?(:scroll_panels).should be_true
    end
      it 'スクコマはコマを含んでいる' do
        r = Scroll.list_opt
        r[:scroll_panels].has_key?(:panel).should be_true
      end
    it '作家を含んでいる' do
      r = Scroll.list_opt
      r.has_key?(:author).should be_true
    end
  end
  describe 'json一覧出力オプションに於いて' do
    before do
      @op = FactoryGirl.create :original_picture, :artist_id => @artist.id
      @p = FactoryGirl.create :picture, :original_picture_id => @op.id, :license_id => @license.id, :artist_id => @artist.id
      @rp = FactoryGirl.create :resource_picture, :artist_id => @artist.id, :license_id => @license.id, :original_picture_id => @op.id, :picture_id => @p.id
      @sbt = FactoryGirl.create :speech_balloon_template
      @scroll = FactoryGirl.create :scroll, :author_id => @author.id, :visible => 1
      @panel = FactoryGirl.create :panel, :author_id => @author.id, :publish => 1
      @scroll_panel = FactoryGirl.create :scroll_panel, :author_id => @author.id, :scroll_id => @scroll.id, :panel_id => @panel.id
    end
    it 'スクコマを含んでいる' do
      r = Scroll.list.to_json Scroll.list_json_opt
      j = JSON.parse r
      i = j.first
      i.has_key?('scroll_panels').should be_true
    end
      it 'スクコマはコマを含んでいる' do
        r = Scroll.list.to_json Scroll.list_json_opt
        j = JSON.parse r
        i = j.first
        s = i['scroll_panels'].first
        s.has_key?('panel').should be_true
      end
    it '作家を含んでいる' do
      r = Scroll.list.to_json Scroll.list_json_opt
      j = JSON.parse r
      i = j.first
      i.has_key?('author').should be_true
    end
  end
  
  describe '単体取得に於いて' do
    before do
      @scroll = FactoryGirl.create :scroll, :author_id => @author.id
    end
    context 'つつがなく終わるとき' do
      it '単体取得オプションを利用している' do
        Scroll.stub(:show_opt).with(any_args).and_return({})
        Scroll.should_receive(:show_opt).with(any_args).exactly(1)
        r = Scroll.show @scroll.id, @author
      end
      it '閲覧許可を問い合わせている' do
        Scroll.any_instance.stub(:visible?).with(any_args).and_return(true)
        Scroll.any_instance.should_receive(:visible?).with(any_args).exactly(1)
        r = Scroll.show @scroll.id, @author
      end
    end
    it '指定のスクロールを返す' do
      c = Scroll.show @scroll.id, @author
      c.should eq @scroll
    end
    context '閲覧許可が出なかったとき' do
      it '403Forbidden例外を返す' do
        Scroll.any_instance.stub(:visible?).and_return(false)
        lambda{
          Scroll.show @scroll.id, @author
        }.should raise_error(ActiveRecord::Forbidden)
      end
    end
    context '存在しないスクロールを開こうとしたとき' do
      it '404RecordNotFound例外を返す' do
        lambda{
          Scroll.show 110, @author
        }.should raise_error(ActiveRecord::RecordNotFound)
      end
    end
  end
  
  describe '編集取得に於いて' do
    before do
      @scroll = FactoryGirl.create :scroll, :author_id => @author.id
    end
    context 'つつがなく終わるとき' do
      it '単体取得オプションを利用している' do
        Scroll.stub(:show_opt).with(any_args).and_return({})
        Scroll.should_receive(:show_opt).with(any_args).exactly(1)
        r = Scroll.edit @scroll.id, @author
      end
      it '所持判定を問い合わせている' do
        Scroll.any_instance.stub(:own?).with(any_args).and_return(true)
        Scroll.any_instance.should_receive(:own?).with(any_args).exactly(1)
        r = Scroll.edit @scroll.id, @author
      end
    end
    it '指定のスクロールを返す' do
      Scroll.any_instance.stub(:own?).and_return(true)
      c = Scroll.edit @scroll.id, @author.id
      c.should eq @scroll
    end
    context '他人のスクロールを開こうとしたとき' do
      it '403Forbidden例外を返す' do
        Scroll.any_instance.stub(:own?).and_return(false)
        lambda{
          Scroll.edit @scroll.id, @author
        }.should raise_error(ActiveRecord::Forbidden)
      end
    end
    context '存在しないスクロールを開こうとしたとき' do
      it '404RecordNotFound例外を返す' do
        lambda{
          Scroll.edit 110, @author
        }.should raise_error(ActiveRecord::RecordNotFound)
      end
    end
  end
  describe '単体取得オプションに於いて' do
    it 'includeキーを含んでいる' do
      r = Scroll.show_opt
      r.has_key?(:include).should be_true
    end
    it '2つの項目を含んでいる' do
      r = Scroll.show_opt[:include]
      r.should have(2).items
    end
    it '作家を含んでいる' do
      r = Scroll.show_opt[:include]
      r.has_key?(:author).should be_true
    end
    it 'スクコマを含んでいる' do
      r = Scroll.show_opt[:include]
      r.has_key?(:scroll_panels).should be_true
    end
      it 'スクコマはコマを含んでいる' do
        r = Scroll.show_opt[:include]
        r[:scroll_panels].has_key?(:panel).should be_true
      end
  end
  describe 'json単体出力オプションに於いて' do
    before do
      @op = FactoryGirl.create :original_picture, :artist_id => @artist.id
      @p = FactoryGirl.create :picture, :original_picture_id => @op.id, :license_id => @license.id, :artist_id => @artist.id
      @rp = FactoryGirl.create :resource_picture, :artist_id => @artist.id, :license_id => @license.id, :original_picture_id => @op.id, :picture_id => @p.id
      @sbt = FactoryGirl.create :speech_balloon_template
      @scroll = FactoryGirl.create :scroll, :author_id => @author.id, :visible => 1
      @panel = FactoryGirl.create :panel, :author_id => @author.id, :publish => 1
      @scroll_panel = FactoryGirl.create :scroll_panel, :author_id => @author.id, :scroll_id => @scroll.id, :panel_id => @panel.id
    end
    it 'スクコマを含んでいる' do
      r = Scroll.show(@scroll.id, @author).to_json Scroll.show_json_opt
      j = JSON.parse r
      i = j
      i.has_key?('scroll_panels').should be_true
    end
      it 'スクコマはコマを含んでいる' do
        r = Scroll.show(@scroll.id, @author).to_json Scroll.show_json_opt
        j = JSON.parse r
        i = j
        s = i['scroll_panels'].first
        s.has_key?('panel').should be_true
      end
    it '作家を含んでいる' do
      r = Scroll.show(@scroll.id, @author).to_json Scroll.show_json_opt
      j = JSON.parse r
      i = j
      i.has_key?('author').should be_true
    end
  end
  
  describe '削除に於いて' do
    before do
      @scroll = FactoryGirl.create :scroll, :author_id => @author.id
      @panel = FactoryGirl.create :panel, :author_id => @author.id, :publish => 1
      @scroll_panel = FactoryGirl.create :scroll_panel, :author_id => @author.id, :scroll_id => @scroll.id, :panel_id => @panel.id
      @other_scroll = FactoryGirl.create :scroll, :author_id => @author.id
      @other_scroll_panel = FactoryGirl.create :scroll_panel, :author_id => @author.id, :scroll_id => @other_scroll.id, :panel_id => @panel.id
    end
    context 'つつがなく終わるとき' do
      it '自身を削除する' do
        lambda {
          r = @scroll.destroy_with_scroll_panel
        }.should change(Scroll, :count).by(-1)
        lambda {
          r = Scroll.find @scroll.id
        }.should raise_error
      end
      it '自身にリンクしているスクコマをすべて削除する' do
        lambda {
          r = @scroll.destroy_with_scroll_panel
        }.should change(ScrollPanel, :count).by(-1)
        lambda {
          r = ScrollPanel.find @scroll_panel.id
        }.should raise_error
      end
      it 'Trueを返す' do
        r = @scroll.destroy_with_scroll_panel
        r.should be_true
      end
    end
    context '削除に失敗したとき' do
      before do
        ScrollPanel.any_instance.stub(:destroy).with(any_args).and_return(false)
      end
      it 'Falseを返す' do
        r = @scroll.destroy_with_scroll_panel
        r.should be_false
      end
      it 'ロールバックしている' do
        lambda {
          r = @scroll.destroy_with_scroll_panel
        }.should_not change(Scroll, :count)
        lambda {
          r = @scroll.destroy_with_scroll_panel
        }.should_not change(ScrollPanel, :count)
      end
    end
  end
end
