# -*- encoding: utf-8 -*-
require 'spec_helper'
#ストーリー

describe Story do
  before do
    @admin = FactoryGirl.create :admin
    @demand_user = FactoryGirl.create :demand_user
    @sp = FactoryGirl.create :system_picture
    @lg = FactoryGirl.create :license_group
    @license = FactoryGirl.create :license, :license_group_id => @lg.id, :system_picture_id => @sp.id
    @user = FactoryGirl.create( :user_yas)
    @author = FactoryGirl.create :author, :user_id => @user.id
    @artist = FactoryGirl.create :artist_yas, :author_id => @author.id
    @other_user = FactoryGirl.create( :user_yas)
    @other_author = FactoryGirl.create :author, :user_id => @other_user.id
    @other_artist = FactoryGirl.create :artist_yas, :author_id => @other_author.id
  end
  
  describe '検証に於いて' do
    before do
      @comic = FactoryGirl.create :comic, :author_id => @author.id
      @story = FactoryGirl.build :story, :comic_id => @comic.id, :author_id => @author.id
    end
    
    context 'オーソドックスなデータのとき' do
      it '下限データが通る' do
        @story.title = 'a'
        @story.visible = 0
        @story.t = 0
        @story.should be_valid
      end
      it '上限データが通る' do
        @story.title = 'a'*100
        @story.visible = 1
        @story.t = 99999
        @story.should be_valid
      end
    end
    
    context 'titleを検証するとき' do
      it 'nullなら失敗する' do
        @story.title = nil
        @story.should_not be_valid
      end
      it '100文字以上なら失敗する' do
        @story.title = 'a'*101
        @story.should_not be_valid
      end
    end
    context 'visibleを検証するとき' do
      it 'nullなら失敗する' do
        @story.visible = nil
        @story.should_not be_valid
      end
      it '負なら失敗する' do
        @story.visible = -1
        @story.should_not be_valid
      end
      it '2以上なら失敗する' do
        @story.visible = 2
        @story.should_not be_valid
      end
    end
    context 'tを検証するとき' do
      it 'nullなら失敗する' do
        @story.t = nil
        @story.should_not be_valid
      end
      it '数値でなければ失敗する' do
        @story.t = 'a'
        @story.should_not be_valid
      end
      it '負なら失敗する' do
        @story.t = -1
        @story.should_not be_valid
      end
    end
  end
  
  describe '文字コード検証に於いて' do
    before do
      @comic = FactoryGirl.create :comic, :author_id => @author.id
      @story = FactoryGirl.build :story, :comic_id => @comic.id, :author_id => @author.id
    end
    
    context 'titleを検証するとき' do
      it 'Shift JISなら失敗する' do
        @story.title = "\x83G\x83r\x83]\x83D"
        lambda{
          @story.valid_encode
        }.should raise_error(Pettanr::BadRequest)
      end
    end
    
    context 'descriptionを検証するとき' do
      it 'Shift JISなら失敗する' do
        @story.description = "\x83G\x83r\x83]\x83D"
        lambda{
          @story.valid_encode
        }.should raise_error(Pettanr::BadRequest)
      end
    end
  end
  
  describe 'デフォルト値補充に於いて' do
    it 'visibleが0になっている' do
      @story = FactoryGirl.build :story, :visible => nil
      @story.supply_default
      @story.visible.should eq 0
    end
  end
  
  describe '上書き補充に於いて' do
    it '作家idが設定されている' do
      @story = FactoryGirl.build :story, :author_id => nil
      @story.overwrite @author
      @story.author_id.should eq @author.id
    end
  end
  
  describe '所持判定に於いて' do
    before do
      @story = FactoryGirl.build :story, :author_id => @author.id
    end
    context '事前チェックする' do
      it '自身にロールリストからの作家取得を依頼している' do
        Story.should_receive(:get_author_from_roles).with(any_args).exactly(1)
        r = @story.own?([@author])
      end
    end
    context 'ロール内作家が取得できるとき' do
      before do
      end
      it 'ロール内作家のidが自身の作家idと一致するなら許可する' do
        Story.stub(:get_author_from_roles).with(any_args).and_return(@author)
        r = @story.own?([@author])
        r.should be_true
      end
      it 'ロール内作家のidが自身の作家idと一致しないならno' do
        Story.stub(:get_author_from_roles).with(any_args).and_return(@other_author)
        @story.own?(@other_author).should be_false
      end
    end
    context 'ロール内作家が取得できないとき' do
      before do
        Story.stub(:get_author_from_roles).with(any_args).and_return(nil)
      end
      it 'Falseを返す' do
        r = @story.own?([@author])
        r.should be_false
      end
    end
  end
  
  describe '閲覧許可に於いて' do
    before do
      @story = FactoryGirl.build :story, :author_id => @author.id
    end
    context 'オープンモードのとき' do
      before do
        MagicNumber['run_mode'] = 0
      end
      it '自身にゲスト用ロールチェックを問い合わせしている' do
        Story.any_instance.stub(:guest_role_check).and_return(true)
        Story.any_instance.should_receive(:guest_role_check).with(any_args).exactly(1)
        r = @story.visible?([@author])
      end
      it 'ゲスト用ロールチェックが失敗したとき、falseを返す' do
        Story.any_instance.stub(:guest_role_check).and_return(false)
        r = @story.visible?([@author])
        r.should be_false
      end
    end
    context 'クローズドモードのとき' do
      before do
        MagicNumber['run_mode'] = 1
      end
      it '自身に読者用ロールチェックを問い合わせしている' do
        Story.any_instance.stub(:reader_role_check).and_return(true)
        Story.any_instance.should_receive(:reader_role_check).with(any_args).exactly(1)
        r = @story.visible?([@author])
      end
      it '読者用ロールチェックが失敗したとき、falseを返す' do
        Story.any_instance.stub(:reader_role_check).and_return(false)
        r = @story.visible?([@author])
        r.should be_false
      end
    end
    context '事前チェックする' do
      before do
        MagicNumber['run_mode'] = 1
        Story.any_instance.stub(:reader_role_check).and_return(true)
        Story.any_instance.stub(:own?).and_return(true)
      end
      it '自身に所持判定を問い合わせしている' do
        Story.any_instance.should_receive(:own?).with(any_args).exactly(1)
        r = @story.visible?([@author])
      end
    end
    context 'つつがなく終わるとき' do
      before do
        MagicNumber['run_mode'] = 1
        Story.any_instance.stub(:reader_role_check).and_return(true)
      end
      it '自分のストーリーなら許可する' do
        Story.any_instance.stub(:own?).and_return(true)
        Story.any_instance.stub(:visible).and_return(0)
        r = @story.visible?([@author])
        r.should be_true
      end
      it '他人の非公開ストーリーなら許可しない' do
        Story.any_instance.stub(:own?).and_return(false)
        Story.any_instance.stub(:visible).and_return(0)
        r = @story.visible?([@author])
        r.should be_false
      end
      it '他人のストーリーでも公開なら許可する' do
        Story.any_instance.stub(:own?).and_return(false)
        Story.any_instance.stub(:visible).and_return(1)
        r = @story.visible?([@author])
        r.should be_true
      end
    end
  end
  
  describe '一覧取得に於いて' do
    before do
      @comic = FactoryGirl.create :comic, :author_id => @author.id
      @story = FactoryGirl.create :story, :comic_id => @comic.id, :author_id => @author.id
    end
    context 'page補正について' do
      it '文字列から数値に変換される' do
        Story.page('8').should eq 8
      end
      it 'nilの場合は1になる' do
        Story.page().should eq 1
      end
      it '0以下の場合は1になる' do
        Story.page('0').should eq 1
      end
    end
    context 'page_size補正について' do
      it '文字列から数値に変換される' do
        Story.page_size('7').should eq 7
      end
      it 'nilの場合はStory.default_page_sizeになる' do
        Story.page_size().should eq Story.default_page_size
      end
      it '0以下の場合はStory.default_page_sizeになる' do
        Story.page_size('0').should eq Story.default_page_size
      end
      it 'Story.max_page_sizeを超えた場合はStory.max_page_sizeになる' do
        Story.page_size('1000').should eq Story.max_page_size
      end
    end
    context 'つつがなく終わるとき' do
      it '一覧取得オプションを利用している' do
        Story.stub(:list_opt).with(any_args).and_return({})
        Story.should_receive(:list_opt).with(any_args).exactly(1)
        r = Story.list
      end
    end
    it 'リストを返す' do
      c = Story.list
      c.should eq [@story]
    end
    it '非公開ストーリーは(自分のストーリーであっても)含んでいない' do
      FactoryGirl.create :story, :comic_id => @comic.id, :author_id => @author.id, :visible => 0
      c = Story.list
      c.should eq [@story]
    end
    it '時系列で並んでいる' do
      #公開ストーリーは(他人のストーリーであっても)含んでいる
      other_comic = FactoryGirl.create :comic, :author_id => @other_author.id
      v = FactoryGirl.create :story, :comic_id => other_comic.id, :author_id => @other_author.id, :updated_at => Time.now + 100
      c = Story.list
      c.should eq [v, @story]
    end
    context 'DBに5件あって1ページの件数を2件に変えたとして' do
      before do
        @story2 = FactoryGirl.create :story, :comic_id => @comic.id, :author_id => @author.id, :updated_at => Time.now + 100
        @story3 = FactoryGirl.create :story, :comic_id => @comic.id, :author_id => @author.id, :updated_at => Time.now + 200
        @story4 = FactoryGirl.create :story, :comic_id => @comic.id, :author_id => @author.id, :updated_at => Time.now + 300
        @story5 = FactoryGirl.create :story, :comic_id => @comic.id, :author_id => @author.id, :updated_at => Time.now + 400
        Story.stub(:default_page_size).and_return(2)
      end
      it '通常は2件を返す' do
        c = Story.list
        c.should have(2).items 
      end
      it 'page=1なら末尾2件を返す' do
        #時系列で並んでいる
        c = Story.list(1)
        c.should eq [@story5, @story4]
      end
      it 'page=2なら中間2件を返す' do
        c = Story.list(2)
        c.should eq [@story3, @story2]
      end
      it 'page=3なら先頭1件を返す' do
        c = Story.list(3)
        c.should eq [@story]
      end
    end
  end
  
  describe '自分のストーリー一覧取得に於いて' do
    before do
      @comic = FactoryGirl.create :comic, :author_id => @author.id
      @story = FactoryGirl.create :story, :comic_id => @comic.id, :author_id => @author.id
    end
    context 'つつがなく終わるとき' do
      it '一覧取得オプションを利用している' do
        Story.stub(:list_opt).with(any_args).and_return({})
        Story.should_receive(:list_opt).with(any_args).exactly(1)
        r = Story.mylist @author
      end
    end
    it 'リストを返す' do
      c = Story.mylist @author
      c.should eq [@story]
    end
    it '時系列で並んでいる' do
      nc = FactoryGirl.create :story, :comic_id => @comic.id, :author_id => @author.id, :updated_at => Time.now + 100
      cl = Story.mylist @author
      cl.should eq [nc, @story]
    end
    it '他人のストーリーは公開でも含まない' do
      other_comic = FactoryGirl.create :comic, :author_id => @other_author.id
      nc = FactoryGirl.create :story, :comic_id => other_comic.id, :author_id => @other_author.id, :visible => 1
      cl = Story.mylist @author
      cl.should eq [@story]
    end
    it '自分のストーリーは非公開でも含んでいる' do
      nc = FactoryGirl.create :story, :comic_id => @comic.id, :author_id => @author.id, :visible => 0, :updated_at => Time.now + 100
      cl = Story.mylist @author
      cl.should eq [nc, @story]
    end
    context 'DBに5件あって1ページの件数を2件に変えたとして' do
      before do
        @story2 = FactoryGirl.create :story, :comic_id => @comic.id, :author_id => @author.id, :updated_at => Time.now + 100
        @story3 = FactoryGirl.create :story, :comic_id => @comic.id, :author_id => @author.id, :updated_at => Time.now + 200
        @story4 = FactoryGirl.create :story, :comic_id => @comic.id, :author_id => @author.id, :updated_at => Time.now + 300
        @story5 = FactoryGirl.create :story, :comic_id => @comic.id, :author_id => @author.id, :updated_at => Time.now + 400
      end
      it '通常は2件を返す' do
        c = Story.mylist @author, 1, 2
        c.should have(2).items 
      end
      it 'page=1なら末尾2件を返す' do
        #時系列で並んでいる
        c = Story.mylist(@author, 1, 2)
        c.should eq [@story5, @story4]
      end
      it 'page=2なら中間2件を返す' do
        c = Story.mylist(@author, 2, 2)
        c.should eq [@story3, @story2]
      end
      it 'page=3なら先頭1件を返す' do
        c = Story.mylist(@author, 3, 2)
        c.should eq [@story]
      end
    end
  end
  
  describe '他作家のストーリー一覧取得に於いて' do
    before do
      @comic = FactoryGirl.create :comic, :author_id => @author.id
      @story = FactoryGirl.create :story, :comic_id => @comic.id, :author_id => @author.id
      @other_comic = FactoryGirl.create :comic, :author_id => @other_author.id
      @other_story = FactoryGirl.create :story, :author_id => @other_author.id, :visible => 1
    end
    context 'つつがなく終わるとき' do
      it '一覧取得オプションを利用している' do
        Story.stub(:list_opt).with(any_args).and_return({})
        Story.should_receive(:list_opt).with(any_args).exactly(1)
        r = Story.himlist @other_author
      end
    end
    it '指定した作家のリストを返す' do
      r = Story.himlist @other_author
      r.should eq [@other_story]
    end
    it '時系列で並んでいる' do
      nc = FactoryGirl.create :story, :comic_id => @other_comic.id, :author_id => @other_author.id, :updated_at => Time.now + 100
      r = Story.himlist @other_author
      r.should eq [nc, @other_story]
    end
    it '公開ストーリーに限る ' do
      hidden = FactoryGirl.create :story, :comic_id => @other_comic.id, :author_id => @other_author.id, :visible => 0
      r = Story.himlist @other_author
      r.should eq [@other_story]
    end
    context 'DBに5件あって1ページの件数を2件に変えたとして' do
      before do
        @other_story2 = FactoryGirl.create :story, :comic_id => @other_comic.id, :author_id => @other_author.id, :updated_at => Time.now + 100
        @other_story3 = FactoryGirl.create :story, :comic_id => @other_comic.id, :author_id => @other_author.id, :updated_at => Time.now + 200
        @other_story4 = FactoryGirl.create :story, :comic_id => @other_comic.id, :author_id => @other_author.id, :updated_at => Time.now + 300
        @other_story5 = FactoryGirl.create :story, :comic_id => @other_comic.id, :author_id => @other_author.id, :updated_at => Time.now + 400
      end
      it '通常は2件を返す' do
        c = Story.himlist @other_author, 1, 2
        c.should have(2).items 
      end
      it 'page=1なら末尾2件を返す' do
        #時系列で並んでいる
        c = Story.himlist(@other_author, 1, 2)
        c.should eq [@other_story5, @other_story4]
      end
      it 'page=2なら中間2件を返す' do
        c = Story.himlist(@other_author, 2, 2)
        c.should eq [@other_story3, @other_story2]
      end
      it 'page=3なら先頭1件を返す' do
        c = Story.himlist(@other_author, 3, 2)
        c.should eq [@other_story]
      end
    end
  end
  
  describe 'ストーリー一覧ページ制御に於いて' do
    before do
      Story.stub(:count).with(any_args).and_return(100)
    end
    it 'ページ制御を返す' do
      r = Story.list_paginate 
      r.is_a?(Kaminari::PaginatableArray).should be_true
    end
    it 'ストーリー一覧の取得条件を利用している' do
      Story.stub(:list_where).with(any_args).and_return('')
      Story.should_receive(:list_where).with(any_args).exactly(1)
      r = Story.list_paginate 
    end
    it 'ページ件数10のとき、3ページ目のオフセットは20から始まる' do
      r = Story.list_paginate 3, 10
      r.limit_value.should eq 10
      r.offset_value.should eq 20
    end
  end
  
  describe '自分のストーリー一覧ページ制御に於いて' do
    before do
      Story.stub(:count).with(any_args).and_return(100)
    end
    it 'ページ制御を返す' do
      r = Story.mylist_paginate @author
      r.is_a?(Kaminari::PaginatableArray).should be_true
    end
    it '自分のストーリー一覧の取得条件を利用している' do
      Story.stub(:mylist_where).with(any_args).and_return('')
      Story.should_receive(:mylist_where).with(any_args).exactly(1)
      r = Story.mylist_paginate @author
    end
    it 'ページ件数10のとき、3ページ目のオフセットは20から始まる' do
      r = Story.mylist_paginate @author, 3, 10
      r.limit_value.should eq 10
      r.offset_value.should eq 20
    end
  end
  
  describe '他作家のストーリー一覧ページ制御に於いて' do
    before do
      Story.stub(:count).with(any_args).and_return(100)
    end
    it 'ページ制御を返す' do
      r = Story.himlist_paginate @other_author
      r.is_a?(Kaminari::PaginatableArray).should be_true
    end
    it '他作家のストーリー一覧の取得条件を利用している' do
      Story.stub(:himlist_where).with(any_args).and_return('')
      Story.should_receive(:himlist_where).with(any_args).exactly(1)
      r = Story.himlist_paginate @other_author
    end
    it 'ページ件数10のとき、3ページ目のオフセットは20から始まる' do
      r = Story.himlist_paginate @other_author, 3, 10
      r.limit_value.should eq 10
      r.offset_value.should eq 20
    end
  end
  
  describe '一覧取得オプションに於いて' do
    it '2つの項目を含んでいる' do
      r = Story.list_opt
      r.should have(2).items
    end
    it 'スト紙を含んでいる' do
      r = Story.list_opt
      r.has_key?(:story_sheets).should be_true
    end
      it 'スト紙は用紙を含んでいる' do
        r = Story.list_opt
        r[:story_sheets].has_key?(:sheet).should be_true
      end
      it 'スト紙は作家を含んでいる' do
        r = Story.list_opt
        r[:story_sheets].has_key?(:author).should be_true
      end
    it '作家を含んでいる' do
      r = Story.list_opt
      r.has_key?(:author).should be_true
    end
  end
  describe 'json一覧出力オプションに於いて' do
    before do
      @comic = FactoryGirl.create :comic, :author_id => @author.id
      @story = FactoryGirl.create :story, :comic_id => @comic.id, :author_id => @author.id, :visible => 1
      @sheet = FactoryGirl.create :sheet, :author_id => @author.id
      @story_sheet = FactoryGirl.create :story_sheet, :author_id => @author.id, :story_id => @story.id, :sheet_id => @sheet.id
    end
    it 'スト紙を含んでいる' do
      r = Story.list.to_json Story.list_json_opt
      j = JSON.parse r
      i = j.first
      i.has_key?('story_sheets').should be_true
    end
      it 'スト紙は用紙を含んでいる' do
        r = Story.list.to_json Story.list_json_opt
        j = JSON.parse r
        i = j.first
        s = i['story_sheets'].first
        s.has_key?('sheet').should be_true
      end
      it 'スト紙は作家を含んでいる' do
        r = Story.list.to_json Story.list_json_opt
        j = JSON.parse r
        i = j.first
        s = i['story_sheets'].first
        s.has_key?('author').should be_true
      end
    it '作家を含んでいる' do
      r = Story.list.to_json Story.list_json_opt
      j = JSON.parse r
      i = j.first
      i.has_key?('author').should be_true
    end
  end
  
  describe '単体取得に於いて' do
    before do
      @comic = FactoryGirl.create :comic, :author_id => @author.id
      @story = FactoryGirl.create :story, :comic_id => @comic.id, :author_id => @author.id
    end
    context 'つつがなく終わるとき' do
      it '単体取得オプションを利用している' do
        Story.stub(:show_opt).with(any_args).and_return({})
        Story.should_receive(:show_opt).with(any_args).exactly(1)
        r = Story.show @story.id, @author
      end
      it '閲覧許可を問い合わせている' do
        Story.any_instance.stub(:visible?).with(any_args).and_return(true)
        Story.any_instance.should_receive(:visible?).with(any_args).exactly(1)
        r = Story.show @story.id, @author
      end
    end
    it '指定のストーリーを返す' do
      c = Story.show @story.id, @author
      c.should eq @story
    end
    context '閲覧許可が出なかったとき' do
      it '403Forbidden例外を返す' do
        Story.any_instance.stub(:visible?).and_return(false)
        lambda{
          Story.show @story.id, @author
        }.should raise_error(ActiveRecord::Forbidden)
      end
    end
    context '存在しないストーリーを開こうとしたとき' do
      it '404RecordNotFound例外を返す' do
        lambda{
          Story.show 110, @author
        }.should raise_error(ActiveRecord::RecordNotFound)
      end
    end
  end
  
  describe '編集取得に於いて' do
    before do
      @comic = FactoryGirl.create :comic, :author_id => @author.id
      @story = FactoryGirl.create :story, :comic_id => @comic.id, :author_id => @author.id
    end
    context 'つつがなく終わるとき' do
      it '単体取得オプションを利用している' do
        Story.stub(:show_opt).with(any_args).and_return({})
        Story.should_receive(:show_opt).with(any_args).exactly(1)
        r = Story.edit @story.id, @author
      end
      it '所持判定を問い合わせている' do
        Story.any_instance.stub(:own?).with(any_args).and_return(true)
        Story.any_instance.should_receive(:own?).with(any_args).exactly(1)
        r = Story.edit @story.id, @author
      end
    end
    it '指定のストーリーを返す' do
      Story.any_instance.stub(:own?).and_return(true)
      c = Story.edit @story.id, @author.id
      c.should eq @story
    end
    context '他人のストーリーを開こうとしたとき' do
      it '403Forbidden例外を返す' do
        Story.any_instance.stub(:own?).and_return(false)
        lambda{
          Story.edit @story.id, @author
        }.should raise_error(ActiveRecord::Forbidden)
      end
    end
    context '存在しないストーリーを開こうとしたとき' do
      it '404RecordNotFound例外を返す' do
        lambda{
          Story.edit 110, @author
        }.should raise_error(ActiveRecord::RecordNotFound)
      end
    end
  end
  describe '単体取得オプションに於いて' do
    it 'includeキーを含んでいる' do
      r = Story.show_opt
      r.has_key?(:include).should be_true
    end
    it '2つの項目を含んでいる' do
      r = Story.show_opt[:include]
      r.should have(2).items
    end
    it '作家を含んでいる' do
      r = Story.show_opt[:include]
      r.has_key?(:author).should be_true
    end
    it 'スト紙を含んでいる' do
      r = Story.show_opt[:include]
      r.has_key?(:story_sheets).should be_true
    end
      it 'スト紙は用紙を含んでいる' do
        r = Story.show_opt[:include]
        r[:story_sheets].has_key?(:sheet).should be_true
      end
      it 'スト紙は作家を含んでいる' do
        r = Story.show_opt[:include]
        r[:story_sheets].has_key?(:author).should be_true
      end
  end
  describe 'json単体出力オプションに於いて' do
    before do
      @comic = FactoryGirl.create :comic, :author_id => @author.id
      @story = FactoryGirl.create :story, :comic_id => @comic.id, :author_id => @author.id, :visible => 1
      @sheet = FactoryGirl.create :sheet, :author_id => @author.id
      @story_sheet = FactoryGirl.create :story_sheet, :author_id => @author.id, :story_id => @story.id, :sheet_id => @sheet.id
    end
    it 'スト紙を含んでいる' do
      r = Story.show(@story.id, @author).to_json Story.show_json_opt
      j = JSON.parse r
      i = j
      i.has_key?('story_sheets').should be_true
    end
      it 'スト紙は用紙を含んでいる' do
        r = Story.show(@story.id, @author).to_json Story.show_json_opt
        j = JSON.parse r
        i = j
        s = i['story_sheets'].first
        s.has_key?('sheet').should be_true
      end
      it 'スト紙は作家を含んでいる' do
        r = Story.show(@story.id, @author).to_json Story.show_json_opt
        j = JSON.parse r
        i = j
        s = i['story_sheets'].first
        s.has_key?('author').should be_true
      end
    it '作家を含んでいる' do
      r = Story.show(@story.id, @author).to_json Story.show_json_opt
      j = JSON.parse r
      i = j
      i.has_key?('author').should be_true
    end
  end
  
  describe 't補充値に於いて' do
    before do
      @comic = FactoryGirl.create :comic, :author_id => @author.id
    end
    
    context 'コミック初のコマなら' do
      it '0を補充値とする' do
        @story = FactoryGirl.build :story, :author_id => @author.id, :comic_id => @comic.id
        @story.t = nil
        r = Story.new_t @story.comic_id
        r.should eq 0
      end
    end
    context 'コミックに一個コマがあるとき' do
      it '1を補充値とする' do
        FactoryGirl.create :story, :author_id => @author.id, :comic_id => @comic.id, :t => 0
        @story = FactoryGirl.build :story, :author_id => @author.id, :comic_id => @comic.id
        @story.t = nil
        r = Story.new_t @story.comic_id
        r.should eq 1
      end
    end
    context 'コミックに2個コマがあるとき' do
      it '2を補充値とする' do
        FactoryGirl.create :story, :author_id => @author.id, :comic_id => @comic.id, :t => 0
        FactoryGirl.create :story, :author_id => @author.id, :comic_id => @comic.id, :t => 1
        @story = FactoryGirl.build :story, :author_id => @author.id, :comic_id => @comic.id
        @story.t = nil
        r = Story.new_t @story.comic_id
        r.should eq 2
      end
    end
  end
  describe 'シリアライズチェックに於いて' do
    context 'つつがなく終わるとき' do
      it '0からシリアライズされているならTrueを返す' do
        r = Story.serial? [0, 1, 2]
        r.should be_true
      end
      it '見た目はシリアライズされてなくてもソート結果が無事ならtrueを返す' do
        r = Story.serial? [0, 2, 1]
        r.should be_true
      end
      it '見た目はシリアライズされてなくてもソート結果が無事ならtrueを返す' do
        r = Story.serial? [ 2, 1, 4, 3, 0]
        r.should be_true
      end
    end
    context '異常なとき' do
      it '0から始まらないならFalseを返す' do
        r = Story.serial? [1, 2, 3]
        r.should be_false
      end
      it '連続していないならFalseを返す' do
        r = Story.serial? [0, 1, 2, 4]
        r.should be_false
      end
      it '連続していないならFalseを返す' do
        r = Story.serial? [0, 1, 2, 4, 5]
        r.should be_false
      end
    end
  end
  describe 't収集に於いて' do
    before do
      @comic = FactoryGirl.create :comic, :author_id => @author.id
      @story = FactoryGirl.create :story, :t => 0, :comic_id => @comic.id, :author_id => @author.id
      @comic2 = FactoryGirl.create :comic, :author_id => @author.id
      @c2story = FactoryGirl.create :story, :t => 0, :comic_id => @comic2.id, :author_id => @author.id
    end
    context 'つつがなく終わるとき' do
      it 'ストーリーから同一コミックのtだけを収集している' do
        r = Story.collect_t @story
        r.should eq [0]
      end
    end
    context '複数コマのとき' do
      it 'ストーリーから同一コミックのtだけを収集している' do
        @story2 = FactoryGirl.create :story, :t => 1, :comic_id => @comic.id, :author_id => @author.id
        r = Story.collect_t @story
        r.sort.should eq [0, 1]
      end
    end
    context '複数ストーリーでヨソのコミックも混じっているとき' do
      it 'ストーリーから同一コミックのtだけを収集している' do
        @story2 = FactoryGirl.create :story, :t => 1, :comic_id => @comic.id, :author_id => @author.id
        r = Story.collect_t @story
        r.sort.should eq [0, 1]
      end
    end
  end
  describe 'tチェックに於いて' do
    before do
      @comic = FactoryGirl.create :comic, :author_id => @author.id
      @story = FactoryGirl.build :story, :t => 1, :comic_id => @comic.id, :author_id => @author.id
    end
    context 'つつがなく終わるとき' do
      it 't収集を依頼している' do
        Story.should_receive(:collect_t).with(any_args).exactly(1)
        Story.stub(:collect_t).with(any_args).and_return([])
        Story.stub(:serial?).with(any_args).and_return(true)
        r = Story.validate_t @story
      end
      it '収集したtをシリアライズチェック依頼している' do
        Story.stub(:collect_t).with(any_args).and_return([])
        Story.should_receive(:serial?).with(any_args).exactly(1)
        Story.stub(:serial?).with(any_args).and_return(true)
        r = Story.validate_t @story
      end
    end
    #実データでチェック
    #依頼チェックだけでは不安なので最低限のチェックを
    context '新規のとき' do
      it '一件だけで正常通過している' do
        @story = FactoryGirl.build :story, :comic_id => @comic.id, :author_id => @author.id, :t => 0
        r = Story.validate_t @story
        r.should be_true 
      end
    end
    context '既存のとき' do
      it '2件目を作っても正常通過している' do
        @story = FactoryGirl.create :story, :comic_id => @comic.id, :author_id => @author.id, :t => 0
        @story2 = FactoryGirl.build :story, :comic_id => @comic.id, :author_id => @author.id, :t => 1
        r = Story.validate_t @story2
        r.should be_true 
      end
    end
  end
  describe '挿入シフトに於いて' do
    before do
      @comic = FactoryGirl.create :comic, :author_id => @author.id
    end
    context '依頼チェック' do
      #テーブルが空で0に挿入
      it 'Updateを依頼している' do
        Story.stub(:update_all).with(any_args)
        Story.should_receive(:update_all).with(any_args).exactly(1)
        @story = FactoryGirl.build :story, :t => 0, :comic_id => @comic.id, :author_id => @author.id
        @story.insert_shift
      end
    end
    context 'テーブルに1件(t:0)で0に挿入したとき' do
      before do
        @story = FactoryGirl.create :story, :t => 0, :comic_id => @comic.id, :author_id => @author.id
        @story2 = FactoryGirl.build :story, :t => 0, :comic_id => @comic.id, :author_id => @author.id
      end
      it '既存の行を1にシフトしている' do
        @story2.insert_shift
        l = Story.find :all
        l.first.t.should eq 1
      end
      it 'これから挿入するt(0)が欠番になっている' do
        @story2.insert_shift
        l = Story.find(:all).map {|s| s.t }
        l.include?(0).should_not be_true
      end
    end
    context 'テーブルに2件(t:0,1)で1に挿入したとき' do
      before do
        @story = FactoryGirl.create :story, :t => 0, :comic_id => @comic.id, :author_id => @author.id
        @story2 = FactoryGirl.create :story, :t => 1, :comic_id => @comic.id, :author_id => @author.id
        @story3 = FactoryGirl.build :story, :t => 1, :comic_id => @comic.id, :author_id => @author.id
      end
      it '既存のt1を2にシフトしてこれから挿入するt(1)が欠番になっている' do
        @story3.insert_shift
        l = Story.find(:all).map {|s| s.t }
        l.sort.should eq [0, 2]
      end
    end
    context 'テーブルに5件(t:0,1,2,3,4)で2に挿入したとき' do
      before do
        @story = FactoryGirl.create :story, :t => 0, :comic_id => @comic.id, :author_id => @author.id
        @story2 = FactoryGirl.create :story, :t => 1, :comic_id => @comic.id, :author_id => @author.id
        @story3 = FactoryGirl.create :story, :t => 2, :comic_id => @comic.id, :author_id => @author.id
        @story4 = FactoryGirl.create :story, :t => 3, :comic_id => @comic.id, :author_id => @author.id
        @story5 = FactoryGirl.create :story, :t => 4, :comic_id => @comic.id, :author_id => @author.id
        @story6 = FactoryGirl.build :story, :t => 2, :comic_id => @comic.id, :author_id => @author.id
      end
      it '既存のt1を2にシフトしてこれから挿入するt(1)が欠番になっている' do
        @story6.insert_shift
        l = Story.find(:all).map {|s| s.t }
        l.sort.should eq [0, 1, 3, 4, 5]
      end
    end
    context '先ほどのケース+他のコミック1件で挿入したとき' do
      before do
        @comic2 = FactoryGirl.create :comic, :author_id => @author.id
        @storyc2 = FactoryGirl.create :story, :t => 0, :comic_id => @comic2.id, :author_id => @author.id
        @story = FactoryGirl.create :story, :t => 0, :comic_id => @comic.id, :author_id => @author.id
        @story2 = FactoryGirl.create :story, :t => 1, :comic_id => @comic.id, :author_id => @author.id
        @story3 = FactoryGirl.create :story, :t => 2, :comic_id => @comic.id, :author_id => @author.id
        @story4 = FactoryGirl.create :story, :t => 3, :comic_id => @comic.id, :author_id => @author.id
        @story5 = FactoryGirl.create :story, :t => 4, :comic_id => @comic.id, :author_id => @author.id
        @story6 = FactoryGirl.build :story, :t => 2, :comic_id => @comic.id, :author_id => @author.id
      end
      it '既存のt1を2にシフトしてこれから挿入するt(1)が欠番になっている' do
        @story6.insert_shift
        l = Story.find(:all, :conditions => ['comic_id = ?', @comic.id]).map {|s| s.t }
        l.sort.should eq [0, 1, 3, 4, 5]
      end
      it '他のコミックに影響がない' do
        ot = @storyc2.t
        @story6.insert_shift
        @storyc2.reload
        @storyc2.t.should eq ot
      end
    end
  end
  describe '少ない方に移動に於いて' do
    before do
      @comic = FactoryGirl.create :comic, :author_id => @author.id
    end
    context '依頼チェック' do
      it 'Updateを依頼している' do
        @story = FactoryGirl.create :story, :t => 0, :comic_id => @comic.id, :author_id => @author.id
        @story2 = FactoryGirl.create :story, :t => 1, :comic_id => @comic.id, :author_id => @author.id
        Story.stub(:update_all).with(any_args)
        Story.should_receive(:update_all).with(any_args).exactly(1)
        ot = @story2.t
        @story2.t = 0
        @story2.lesser_shift ot
      end
    end
    context 'テーブルに2件(t:0,1)で1を0に移動したとき' do
      before do
        @story = FactoryGirl.create :story, :t => 0, :comic_id => @comic.id, :author_id => @author.id
        @story2 = FactoryGirl.create :story, :t => 1, :comic_id => @comic.id, :author_id => @author.id
        @ot = @story2.t
        @story2.t = 0
      end
      it '既存のt0を1にシフトしてこれから挿入するt(0)が欠番になっている' do
        #移動させたい行はそのまま残る
        @story2.lesser_shift @ot
        l = Story.find(:all).map {|s| s.t }
        l.sort.should eq [1, 1]
      end
      it '既存のt0を1にシフトしている' do
        @story2.lesser_shift @ot
        @story.reload
        @story.t.should eq 1
      end
    end
    context 'テーブルに3件(t:0,1,2)で2を1に移動したとき' do
      before do
        @story = FactoryGirl.create :story, :t => 0, :comic_id => @comic.id, :author_id => @author.id
        @story2 = FactoryGirl.create :story, :t => 1, :comic_id => @comic.id, :author_id => @author.id
        @story3 = FactoryGirl.create :story, :t => 2, :comic_id => @comic.id, :author_id => @author.id
        @ot = @story3.t
        @story3.t = 1
      end
      it '既存のt1を2にシフトしてこれから挿入するt(1)が欠番になっている' do
        #移動させたい行はそのまま残る
        @story3.lesser_shift @ot
        l = Story.find(:all).map {|s| s.t }
        l.sort.should eq [0, 2, 2]
      end
      it '既存のt1を2にシフトしている' do
        @story3.lesser_shift @ot
        @story2.reload
        @story2.t.should eq 2
      end
    end
    context 'テーブルに5件(t:0,1,2,3,4)で3を1に移動したとき' do
      before do
        @story = FactoryGirl.create :story, :t => 0, :comic_id => @comic.id, :author_id => @author.id
        @story2 = FactoryGirl.create :story, :t => 1, :comic_id => @comic.id, :author_id => @author.id
        @story3 = FactoryGirl.create :story, :t => 2, :comic_id => @comic.id, :author_id => @author.id
        @story4 = FactoryGirl.create :story, :t => 3, :comic_id => @comic.id, :author_id => @author.id
        @story5 = FactoryGirl.create :story, :t => 4, :comic_id => @comic.id, :author_id => @author.id
        @ot = @story4.t
        @story4.t = 1
      end
      it 'これから挿入するt(1)が欠番になっている' do
        #移動させたい行はそのまま残る
        @story4.lesser_shift @ot
        l = Story.find(:all).map {|s| s.t }
        l.sort.should eq [0, 2, 3, 3, 4]
      end
      it '既存のt0には変化がない' do
        @story4.lesser_shift @ot
        @story.reload
        @story.t.should eq 0
      end
      it '既存のt4には変化がない' do
        @story4.lesser_shift @ot
        @story5.reload
        @story5.t.should eq 4
      end
      it '既存のt1を2にシフトしている' do
        @story4.lesser_shift @ot
        @story2.reload
        @story2.t.should eq 2
      end
      it '既存のt2を3にシフトしている' do
        @story4.lesser_shift @ot
        @story3.reload
        @story3.t.should eq 3
      end
    end
    context '先ほどのケース+他のコミック1件で挿入したとき' do
      before do
        @comic2 = FactoryGirl.create :comic, :author_id => @author.id
        @storyc2 = FactoryGirl.create :story, :t => 0, :comic_id => @comic2.id, :author_id => @author.id
        @story = FactoryGirl.create :story, :t => 0, :comic_id => @comic.id, :author_id => @author.id
        @story2 = FactoryGirl.create :story, :t => 1, :comic_id => @comic.id, :author_id => @author.id
        @story3 = FactoryGirl.create :story, :t => 2, :comic_id => @comic.id, :author_id => @author.id
        @story4 = FactoryGirl.create :story, :t => 3, :comic_id => @comic.id, :author_id => @author.id
        @story5 = FactoryGirl.create :story, :t => 4, :comic_id => @comic.id, :author_id => @author.id
        @ot = @story4.t
        @story4.t = 1
      end
      it 'これから挿入するt(1)が欠番になっている' do
        @story4.lesser_shift @ot
        l = Story.find(:all).map {|s| s.t }
        l.sort.should eq [0, 0, 2, 3, 3, 4]
      end
      it '既存のt0には変化がない' do
        @story4.lesser_shift @ot
        @story.reload
        @story.t.should eq 0
      end
      it '既存のt4には変化がない' do
        @story4.lesser_shift @ot
        @story5.reload
        @story5.t.should eq 4
      end
      it '既存のt1を2にシフトしている' do
        @story4.lesser_shift @ot
        @story2.reload
        @story2.t.should eq 2
      end
      it '既存のt2を3にシフトしている' do
        @story4.lesser_shift @ot
        @story3.reload
        @story3.t.should eq 3
      end
      it '他のコミックに影響がない' do
        @story4.lesser_shift @ot
        @storyc2.reload
        @storyc2.t.should eq 0
      end
    end
    #例外ケース。
    #負のときは0として正常扱い
    context 'テーブルに2件(t:0,1)で1を-1に移動したとき' do
      before do
        @story = FactoryGirl.create :story, :t => 0, :comic_id => @comic.id, :author_id => @author.id
        @story2 = FactoryGirl.create :story, :t => 1, :comic_id => @comic.id, :author_id => @author.id
        @ot = @story2.t
        @story2.t = -1
      end
      it '既存のt0を1にシフトしてこれから挿入するt(0)が欠番になっている' do
        #移動させたい行はそのまま残る
        @story2.lesser_shift @ot
        l = Story.find(:all).map {|s| s.t }
        l.sort.should eq [1, 1]
      end
      it '既存のt0を1にシフトしている' do
        @story2.lesser_shift @ot
        @story.reload
        @story.t.should eq 1
      end
      it '既存のt1は0に補正されている' do
        @story2.lesser_shift @ot
        @story2.t.should eq 0
      end
    end
  end
  describe '大きい方に移動に於いて' do
    before do
      @comic = FactoryGirl.create :comic, :author_id => @author.id
    end
    context '依頼チェック' do
      it 'Updateを依頼している' do
        @story = FactoryGirl.create :story, :t => 0, :comic_id => @comic.id, :author_id => @author.id
        @story2 = FactoryGirl.create :story, :t => 1, :comic_id => @comic.id, :author_id => @author.id
        Story.stub(:update_all).with(any_args)
        Story.should_receive(:update_all).with(any_args).exactly(1)
        ot = @story.t
        @story.t = 1
        @story.higher_shift ot
      end
    end
    context 'テーブルに2件(t:0,1)で0を1に移動したとき' do
      before do
        @story = FactoryGirl.create :story, :t => 0, :comic_id => @comic.id, :author_id => @author.id
        @story2 = FactoryGirl.create :story, :t => 1, :comic_id => @comic.id, :author_id => @author.id
        @ot = @story.t
        @story.t = 1
      end
      it '既存のt1を0にシフトしてこれから挿入するt(1)が欠番になっている' do
        #移動させたい行はそのまま残る
        @story.higher_shift @ot
        l = Story.find(:all).map {|s| s.t }
        l.sort.should eq [0, 0]
      end
      it '既存のt1を0にシフトしている' do
        @story.higher_shift @ot
        @story2.reload
        @story2.t.should eq 0
      end
    end
    context 'テーブルに3件(t:0,1,2)で0を1に移動したとき' do
      before do
        @story = FactoryGirl.create :story, :t => 0, :comic_id => @comic.id, :author_id => @author.id
        @story2 = FactoryGirl.create :story, :t => 1, :comic_id => @comic.id, :author_id => @author.id
        @story3 = FactoryGirl.create :story, :t => 2, :comic_id => @comic.id, :author_id => @author.id
        @ot = @story.t
        @story.t = 1
      end
      it '既存のt1を0にシフトしてこれから挿入するt(1)が欠番になっている' do
        #移動させたい行はそのまま残る
        @story.higher_shift @ot
        l = Story.find(:all).map {|s| s.t }
        l.sort.should eq [0, 0, 2]
      end
      it '既存のt1を0にシフトしている' do
        @story.higher_shift @ot
        @story2.reload
        @story2.t.should eq 0
      end
    end
    context 'テーブルに5件(t:0,1,2,3,4)で1を3に移動したとき' do
      before do
        @story = FactoryGirl.create :story, :t => 0, :comic_id => @comic.id, :author_id => @author.id
        @story2 = FactoryGirl.create :story, :t => 1, :comic_id => @comic.id, :author_id => @author.id
        @story3 = FactoryGirl.create :story, :t => 2, :comic_id => @comic.id, :author_id => @author.id
        @story4 = FactoryGirl.create :story, :t => 3, :comic_id => @comic.id, :author_id => @author.id
        @story5 = FactoryGirl.create :story, :t => 4, :comic_id => @comic.id, :author_id => @author.id
        @ot = @story2.t
        @story2.t = 3
      end
      it 'これから挿入するt(3)が欠番になっている' do
        #移動させたい行はそのまま残る
        @story2.higher_shift @ot
        l = Story.find(:all).map {|s| s.t }
        l.sort.should eq [0, 1, 1, 2, 4]
      end
      it '既存のt0には変化がない' do
        @story2.higher_shift @ot
        @story.reload
        @story.t.should eq 0
      end
      it '既存のt4には変化がない' do
        @story2.higher_shift @ot
        @story5.reload
        @story5.t.should eq 4
      end
      it '既存のt2を1にシフトしている' do
        @story2.higher_shift @ot
        @story3.reload
        @story3.t.should eq 1
      end
      it '既存のt3を2にシフトしている' do
        @story2.higher_shift @ot
        @story4.reload
        @story4.t.should eq 2
      end
    end
    context '先ほどのケース+他のコミック1件で挿入したとき' do
      before do
        @comic2 = FactoryGirl.create :comic, :author_id => @author.id
        @storyc2 = FactoryGirl.create :story, :t => 0, :comic_id => @comic2.id, :author_id => @author.id
        @story = FactoryGirl.create :story, :t => 0, :comic_id => @comic.id, :author_id => @author.id
        @story2 = FactoryGirl.create :story, :t => 1, :comic_id => @comic.id, :author_id => @author.id
        @story3 = FactoryGirl.create :story, :t => 2, :comic_id => @comic.id, :author_id => @author.id
        @story4 = FactoryGirl.create :story, :t => 3, :comic_id => @comic.id, :author_id => @author.id
        @story5 = FactoryGirl.create :story, :t => 4, :comic_id => @comic.id, :author_id => @author.id
        @ot = @story2.t
        @story2.t = 3
      end
      it 'これから挿入するt(3)が欠番になっている' do
        #移動させたい行はそのまま残る
        @story2.higher_shift @ot
        l = Story.find(:all).map {|s| s.t }
        l.sort.should eq [0, 0, 1, 1, 2, 4]
      end
      it '既存のt0には変化がない' do
        @story2.higher_shift @ot
        @story.reload
        @story.t.should eq 0
      end
      it '既存のt4には変化がない' do
        @story2.higher_shift @ot
        @story5.reload
        @story5.t.should eq 4
      end
      it '既存のt2を1にシフトしている' do
        @story2.higher_shift @ot
        @story3.reload
        @story3.t.should eq 1
      end
      it '既存のt3を2にシフトしている' do
        @story2.higher_shift @ot
        @story4.reload
        @story4.t.should eq 2
      end
      it '他のコミックに影響がない' do
        @story2.higher_shift @ot
        @storyc2.reload
        @storyc2.t.should eq 0
      end
    end
    #例外ケース。
    #max超えたときはmaxとして正常扱い
    context 'テーブルに2件(t:0,1)で0を2に移動したとき' do
      before do
        @story = FactoryGirl.create :story, :t => 0, :comic_id => @comic.id, :author_id => @author.id
        @story2 = FactoryGirl.create :story, :t => 1, :comic_id => @comic.id, :author_id => @author.id
        @ot = @story.t
        @story.t = 2
      end
      it '既存のt1を0にシフトしてこれから挿入するt(1)が欠番になっている' do
        #移動させたい行はそのまま残る
        @story.higher_shift @ot
        l = Story.find(:all).map {|s| s.t }
        l.sort.should eq [0, 0]
      end
      it '既存のt1を0にシフトしている' do
        @story.higher_shift @ot
        @story2.reload
        @story2.t.should eq 0
      end
      it '既存のt0は1に補正されている' do
        @story.higher_shift @ot
        @story.t.should eq 1
      end
    end
  end
  describe '入れ替えに於いて' do
    before do
      @comic = FactoryGirl.create :comic, :author_id => @author.id
      @story = FactoryGirl.create :story, :t => 0, :comic_id => @comic.id, :author_id => @author.id
      @story2 = FactoryGirl.create :story, :t => 1, :comic_id => @comic.id, :author_id => @author.id
    end
    context '新tが旧tより小さいとき' do
      it '少ない方に移動を依頼している' do
        Story.any_instance.stub(:lesser_shift).with(any_args)
        Story.any_instance.should_receive(:lesser_shift).with(any_args).exactly(1)
        ot = @story2.t
        @story2.t = 0
        @story2.update_shift ot
      end
    end
    context '新tが旧tより大きいとき' do
      it '大きい方に移動を依頼している' do
        Story.any_instance.stub(:higher_shift).with(any_args)
        Story.any_instance.should_receive(:higher_shift).with(any_args).exactly(1)
        ot = @story.t
        @story.t = 1
        @story.update_shift ot
      end
    end
  end
  describe '順序入れ替えに於いて' do
    before do
      @comic = FactoryGirl.create :comic, :author_id => @author.id
    end
    context 'オブジェクトが新規でtが空のとき' do
      it '末尾追加としてtを補充依頼している' do
        @story = FactoryGirl.build :story, :comic_id => @comic.id, :author_id => @author.id
        Story.stub(:new_t).with(any_args).and_return(0)
        Story.should_receive(:new_t).with(any_args).exactly(1)
        @story.t = nil
        r = @story.rotate
      end
    end
    context 'オブジェクトが新規でtが設定されているとき' do
      it '挿入追加として挿入シフトを依頼している' do
        @story = FactoryGirl.build :story, :comic_id => @comic.id, :author_id => @author.id
        Story.any_instance.stub(:insert_shift).with(any_args)
        Story.any_instance.should_receive(:insert_shift).with(any_args).exactly(1)
        @story.t = 0
        r = @story.rotate
      end
    end
    context 'オブジェクトが新規でなくtが設定されているとき' do
      it '移動として入れ替えを依頼している' do
        @story = FactoryGirl.create :story, :t => 0, :comic_id => @comic.id, :author_id => @author.id
        @story2 = FactoryGirl.create :story, :t => 1, :comic_id => @comic.id, :author_id => @author.id
        Story.any_instance.stub(:update_shift).with(any_args)
        Story.any_instance.should_receive(:update_shift).with(1).exactly(1)
        @story2.t = 0
        r = @story.rotate 1
      end
    end
    context 'オブジェクトが新規でなくtが空のとき' do
      it '入れ替えもシフトもせず、tを空のままにしている' do
        #結果、tに欠番が生じてシリアライズチェックでひっかかる
      end
    end
  end
  describe '編集許可に於いて' do
    before do
      @comic = FactoryGirl.create :comic, :author_id => @author.id
      @panel = FactoryGirl.create :panel, :author_id => @author.id
      @story = FactoryGirl.build :story, :t => nil, :comic_id => @comic.id, :author_id => @author.id
    end
    context 'つつがなく終わるとき' do
      it 'trueを返す' do
        r = @story.allow?
        r.should be_true
      end
    end
    context 'コミックで引っかかるとき' do
      it 'falseを返す' do
        Panel.any_instance.stub(:usable?).with(any_args).and_return(true)
        Comic.any_instance.stub(:own?).with(any_args).and_return(false)
        r = @story.allow?
        r.should be_false
      end
    end
    context 'コミックが指定されていなかったとき' do
      it 'nilを返す' do
        Comic.any_instance.stub(:own?).with(any_args).and_return(true)
        @story.comic_id = nil
        r = @story.allow?
        r.should eq nil
      end
    end
  end
  describe '保存に於いて' do
    before do
      @comic = FactoryGirl.create :comic, :author_id => @author.id
      @story = FactoryGirl.build :story, :t => nil, :comic_id => @comic.id, :author_id => @author.id
    end
    context 'つつがなく終わるとき' do
      it '編集許可チェックを依頼している' do
        Story.any_instance.stub(:allow?).with(any_args).and_return(true)
        Story.any_instance.should_receive(:allow?).with(any_args).exactly(1)
        r = @story.store
      end
      it '順序入れ替えを依頼している' do
        Story.any_instance.stub(:rotate).with(any_args).and_return(0)
        Story.any_instance.should_receive(:rotate).with(any_args).exactly(1)
        Story.any_instance.stub(:save).with(any_args).and_return(true)
        Story.stub(:validate_t).with(any_args).and_return(true)
        r = @story.store 
      end
      it '保存を依頼している' do
        Story.stub(:new_t).with(any_args).and_return(0)
        Story.any_instance.stub(:save).with(any_args).and_return(true)
        Story.any_instance.should_receive(:save).with(any_args).exactly(1)
        Story.stub(:validate_t).with(any_args).and_return(true)
        r = @story.store
      end
      it 'tのシリアライズチェックを依頼している' do
        Story.stub(:new_t).with(any_args).and_return(0)
        Story.any_instance.stub(:save).with(any_args).and_return(true)
        Story.stub(:validate_t).with(any_args).and_return(true)
        Story.should_receive(:validate_t).with(any_args).exactly(1)
        r = @story.store
      end
    end
    #入れ替えテストと同じテストを実施。こちらはシフトだけでなく本尊も更新されている
    context 'テーブルに5件(t:0,1,2,3,4)+他のコミック1件で2に挿入したとき' do
      before do
        @comic2 = FactoryGirl.create :comic, :author_id => @author.id
        @storyc2 = FactoryGirl.create :story, :t => 0, :comic_id => @comic2.id, :author_id => @author.id
        @story = FactoryGirl.create :story, :t => 0, :comic_id => @comic.id, :author_id => @author.id
        @story2 = FactoryGirl.create :story, :t => 1, :comic_id => @comic.id, :author_id => @author.id
        @story3 = FactoryGirl.create :story, :t => 2, :comic_id => @comic.id, :author_id => @author.id
        @story4 = FactoryGirl.create :story, :t => 3, :comic_id => @comic.id, :author_id => @author.id
        @story5 = FactoryGirl.create :story, :t => 4, :comic_id => @comic.id, :author_id => @author.id
        @story6 = FactoryGirl.build :story, :t => 2, :comic_id => @comic.id, :author_id => @author.id
      end
      it '既存のt0には変化がない' do
        @story6.store
        @story.reload
        @story.t.should eq 0
      end
      it '既存のt1には変化がない' do
        @story6.store
        @story2.reload
        @story2.t.should eq 1
      end
      it '既存のt2を3にシフトしている' do
        @story6.store
        @story3.reload
        @story3.t.should eq 3
      end
      it '既存のt3を4にシフトしている' do
        @story6.store
        @story4.reload
        @story4.t.should eq 4
      end
      it '既存のt5を5にシフトしている' do
        @story6.store
        @story5.reload
        @story5.t.should eq 5
      end
      it '新規のt2が作成されている' do
        @story6.store
        @story6.reload
        @story6.t.should eq 2
      end
      it '他のコミックに影響がない' do
        @ot = @storyc2.t
        @story6.store
        @storyc2.reload
        @storyc2.t.should eq @ot
      end
    end
    context 'テーブルに5件(t:0,1,2,3,4)+他のコミック1件で3を1に移動したとき' do
      before do
        @comic2 = FactoryGirl.create :comic, :author_id => @author.id
        @storyc2 = FactoryGirl.create :story, :t => 0, :comic_id => @comic2.id, :author_id => @author.id
        @story = FactoryGirl.create :story, :t => 0, :comic_id => @comic.id, :author_id => @author.id
        @story2 = FactoryGirl.create :story, :t => 1, :comic_id => @comic.id, :author_id => @author.id
        @story3 = FactoryGirl.create :story, :t => 2, :comic_id => @comic.id, :author_id => @author.id
        @story4 = FactoryGirl.create :story, :t => 3, :comic_id => @comic.id, :author_id => @author.id
        @story5 = FactoryGirl.create :story, :t => 4, :comic_id => @comic.id, :author_id => @author.id
        @ot = @story4.t
        @story4.t = 1
      end
      it '既存のt0には変化がない' do
        @story4.store @ot
        @story.reload
        @story.t.should eq 0
      end
      it '既存のt4には変化がない' do
        @story4.store @ot
        @story5.reload
        @story5.t.should eq 4
      end
      it '既存のt1を2にシフトしている' do
        @story4.store @ot
        @story2.reload
        @story2.t.should eq 2
      end
      it '既存のt2を3にシフトしている' do
        @story4.store @ot
        @story3.reload
        @story3.t.should eq 3
      end
      it '既存のt3を1にシフトしている' do
        @story4.store @ot
        @story4.reload
        @story4.t.should eq 1
      end
      it '他のコミックに影響がない' do
        @story4.store @ot
        @storyc2.reload
        @storyc2.t.should eq 0
      end
    end
    context 'テーブルに5件(t:0,1,2,3,4)+他のコミック1件で1を3に移動したとき' do
      before do
        @comic2 = FactoryGirl.create :comic, :author_id => @author.id
        @storyc2 = FactoryGirl.create :story, :t => 0, :comic_id => @comic2.id, :author_id => @author.id
        @story = FactoryGirl.create :story, :t => 0, :comic_id => @comic.id, :author_id => @author.id
        @story2 = FactoryGirl.create :story, :t => 1, :comic_id => @comic.id, :author_id => @author.id
        @story3 = FactoryGirl.create :story, :t => 2, :comic_id => @comic.id, :author_id => @author.id
        @story4 = FactoryGirl.create :story, :t => 3, :comic_id => @comic.id, :author_id => @author.id
        @story5 = FactoryGirl.create :story, :t => 4, :comic_id => @comic.id, :author_id => @author.id
        @ot = @story2.t
        @story2.t = 3
      end
      it '既存のt0には変化がない' do
        @story2.store @ot
        @story.reload
        @story.t.should eq 0
      end
      it '既存のt4には変化がない' do
        @story2.store @ot
        @story5.reload
        @story5.t.should eq 4
      end
      it '既存のt1を3にシフトしている' do
        @story2.store @ot
        @story2.reload
        @story2.t.should eq 3
      end
      it '既存のt2を1にシフトしている' do
        @story2.store @ot
        @story3.reload
        @story3.t.should eq 1
      end
      it '既存のt3を2にシフトしている' do
        @story2.store @ot
        @story4.reload
        @story4.t.should eq 2
      end
      it '他のコミックに影響がない' do
        @story2.store @ot
        @storyc2.reload
        @storyc2.t.should eq 0
      end
    end
    #ロールバックテスト。入れ替えが直接DBをいじるので、すべてのケースで確実にロールバックを確認する
    context 'テーブルに5件(t:0,1,2,3,4)+他のコミック1件で2に挿入したが保存に失敗したとき' do
      before do
        Story.any_instance.stub(:save).with(any_args).and_return(false)
        @comic2 = FactoryGirl.create :comic, :author_id => @author.id
        @storyc2 = FactoryGirl.create :story, :t => 0, :comic_id => @comic2.id, :author_id => @author.id
        @story = FactoryGirl.create :story, :t => 0, :comic_id => @comic.id, :author_id => @author.id
        @story2 = FactoryGirl.create :story, :t => 1, :comic_id => @comic.id, :author_id => @author.id
        @story3 = FactoryGirl.create :story, :t => 2, :comic_id => @comic.id, :author_id => @author.id
        @story4 = FactoryGirl.create :story, :t => 3, :comic_id => @comic.id, :author_id => @author.id
        @story5 = FactoryGirl.create :story, :t => 4, :comic_id => @comic.id, :author_id => @author.id
        @story6 = FactoryGirl.build :story, :t => 2, :comic_id => @comic.id, :author_id => @author.id
      end
      it '既存のtに変化がない' do
        @story6.store
        @story.reload
        @story.t.should eq 0
        @story2.reload
        @story2.t.should eq 1
        @story3.reload
        @story3.t.should eq 2
        @story4.reload
        @story4.t.should eq 3
        @story5.reload
        @story5.t.should eq 4
        @storyc2.reload
        @storyc2.t.should eq 0
      end
      it 'falseを返す' do
        r = @story6.store
        r.should be_false
      end
    end
    context 'テーブルに5件(t:0,1,2,3,4)+他のコミック1件で3を1に移動したがシリアルチェックに失敗したとき' do
      before do
        Story.stub(:validate_t).with(any_args).and_return(false)
        @comic2 = FactoryGirl.create :comic, :author_id => @author.id
        @storyc2 = FactoryGirl.create :story, :t => 0, :comic_id => @comic2.id, :author_id => @author.id
        @story = FactoryGirl.create :story, :t => 0, :comic_id => @comic.id, :author_id => @author.id
        @story2 = FactoryGirl.create :story, :t => 1, :comic_id => @comic.id, :author_id => @author.id
        @story3 = FactoryGirl.create :story, :t => 2, :comic_id => @comic.id, :author_id => @author.id
        @story4 = FactoryGirl.create :story, :t => 3, :comic_id => @comic.id, :author_id => @author.id
        @story5 = FactoryGirl.create :story, :t => 4, :comic_id => @comic.id, :author_id => @author.id
        @ot = @story4.t
        @story4.t = 1
      end
      it '既存のtに変化がない' do
        @story4.store @ot
        @story.reload
        @story.t.should eq 0
        @story2.reload
        @story2.t.should eq 1
        @story3.reload
        @story3.t.should eq 2
        @story4.reload
        @story4.t.should eq 3
        @story5.reload
        @story5.t.should eq 4
        @storyc2.reload
        @storyc2.t.should eq 0
      end
      it 'falseを返す' do
        r = @story4.store @ot
        r.should be_false
      end
      it 'tにエラーメッセージが入っている' do
        @story4.store @ot
        @story4.errors[:t].should_not be_empty
        @story4.valid?.should be_true
      end
    end
    context '編集不可だったとき' do
      before do
        @story = FactoryGirl.build :story, :t => 0, :comic_id => @comic.id, :author_id => @author.id
        Story.any_instance.stub(:allow?).and_return(false)
      end
      it '403Forbidden例外を返す' do
        lambda{
          @story.store
        }.should raise_error(ActiveRecord::Forbidden)
      end
    end
  end
  describe '切り詰め処理つき削除に於いて' do
    before do
      @comic = FactoryGirl.create :comic, :author_id => @author.id
      @story = FactoryGirl.create :story, :t => 0, :comic_id => @comic.id, :author_id => @author.id
    end
    context 'つつがなく終わるとき' do
      it '削除される' do
        lambda{
          @story.destroy_and_shorten
        }.should change(Story, :count ).by(-1)
      end
      it 'Trueを返す' do
        r = @story.destroy_and_shorten
        r.should be_true 
      end
    end
    context '削除に失敗したとき' do
      before do
        Story.any_instance.stub(:destroy).and_return(false)
      end
      it 'ロールバックされる' do
        lambda{
          @story.destroy_and_shorten
        }.should_not change(Story, :count )
      end
      it 'Falseを返す' do
        r = @story.destroy_and_shorten
        r.should be_false
      end
    end
    #連携テスト。切り詰めが直接DBをいじる
    context '2件で先頭を削除したとき' do
      before do
        @story2 = FactoryGirl.create :story, :t => 1, :comic_id => @comic.id, :author_id => @author.id
      end
      it '行が削除される' do
        lambda{
          @story.destroy_and_shorten
        }.should change(Story, :count ).by(-1)
      end
      it '先頭は削除される' do
        @story.destroy_and_shorten
        lambda{
          Story.find @story.id
        }.should raise_error(ActiveRecord::RecordNotFound)
      end
      it '2件目は前に詰められる' do
        @story.destroy_and_shorten
        @story2.reload
        @story2.t.should eq 0
      end
    end
    context '3件で先頭を削除したとき' do
      before do
        @story2 = FactoryGirl.create :story, :t => 1, :comic_id => @comic.id, :author_id => @author.id
        @story3 = FactoryGirl.create :story, :t => 2, :comic_id => @comic.id, :author_id => @author.id
      end
      it '行が削除される' do
        lambda{
          @story.destroy_and_shorten
        }.should change(Story, :count ).by(-1)
      end
      it '先頭は削除される' do
        @story.destroy_and_shorten
        lambda{
          Story.find @story.id
        }.should raise_error(ActiveRecord::RecordNotFound)
      end
      it '2件目は前に詰められる' do
        @story.destroy_and_shorten
        @story2.reload
        @story2.t.should eq 0
      end
      it '3件目は前に詰められる' do
        @story.destroy_and_shorten
        @story3.reload
        @story3.t.should eq 1
      end
    end
    context '5件で3件目を削除したとき' do
      before do
        @story2 = FactoryGirl.create :story, :t => 1, :comic_id => @comic.id, :author_id => @author.id
        @story3 = FactoryGirl.create :story, :t => 2, :comic_id => @comic.id, :author_id => @author.id
        @story4 = FactoryGirl.create :story, :t => 3, :comic_id => @comic.id, :author_id => @author.id
        @story5 = FactoryGirl.create :story, :t => 4, :comic_id => @comic.id, :author_id => @author.id
      end
      it '行が削除される' do
        lambda{
          @story3.destroy_and_shorten
        }.should change(Story, :count ).by(-1)
      end
      it '1件目は変化がない' do
        @story3.destroy_and_shorten
        @story.reload
        @story.t.should eq 0
      end
      it '2件目は変化がない' do
        @story3.destroy_and_shorten
        @story2.reload
        @story2.t.should eq 1
      end
      it '3件目は削除される' do
        @story3.destroy_and_shorten
        lambda{
          Story.find @story3.id
        }.should raise_error(ActiveRecord::RecordNotFound)
      end
      it '4件目は前に詰められる' do
        @story3.destroy_and_shorten
        @story4.reload
        @story4.t.should eq 2
      end
      it '5件目は前に詰められる' do
        @story3.destroy_and_shorten
        @story5.reload
        @story5.t.should eq 3
      end
    end
    #ロールバックテスト。切り詰めが直接DBをいじるので、すべてのケースで確実にロールバックを確認する
  end
  describe '削除に於いて' do
    before do
      @comic = FactoryGirl.create :comic, :author_id => @author.id
      @story = FactoryGirl.create :story, :comic_id => @comic.id, :author_id => @author.id
      @sheet = FactoryGirl.create :sheet, :author_id => @author.id
      @story_sheet = FactoryGirl.create :story_sheet, :author_id => @author.id, :story_id => @story.id, :sheet_id => @sheet.id
      @other_comic = FactoryGirl.create :comic, :author_id => @author.id
      @other_story = FactoryGirl.create :story, :comic_id => @other_comic.id, :author_id => @author.id
      @other_sheet = FactoryGirl.create :sheet, :author_id => @author.id
      @other_story_sheet = FactoryGirl.create :story_sheet, :author_id => @author.id, :story_id => @other_story.id, :sheet_id => @other_sheet.id
    end
    context 'つつがなく終わるとき' do
      it '自身を削除する' do
        lambda {
          r = @story.destroy_with_story_sheet
        }.should change(Story, :count).by(-1)
        lambda {
          r = Story.find @story.id
        }.should raise_error
      end
      it '自身にリンクしているスト紙をすべて削除する' do
        lambda {
          r = @story.destroy_with_story_sheet
        }.should change(StorySheet, :count).by(-1)
        lambda {
          r = StorySheet.find @story_sheet.id
        }.should raise_error
      end
      it 'Trueを返す' do
        r = @story.destroy_with_story_sheet
        r.should be_true
      end
    end
    context '削除に失敗したとき' do
      before do
        StorySheet.any_instance.stub(:destroy).with(any_args).and_return(false)
      end
      it 'Falseを返す' do
        r = @story.destroy_with_story_sheet
        r.should be_false
      end
      it 'ロールバックしている' do
        lambda {
          r = @story.destroy_with_story_sheet
        }.should_not change(Story, :count)
        lambda {
          r = @story.destroy_with_story_sheet
        }.should_not change(StorySheet, :count)
      end
    end
  end
end
